//**************************************************************************
// ﾌｧｲﾙ内容     「aqm1248_sw_con_test.ino」AQM1248液晶_SW_10ピンコネクタ_シールド テストプログラム
// Copyright    ジャパンマイコンカーラリー実行委員会
// ライセンス   This software is released under the MIT License.
//              http://opensource.org/licenses/mit-license.php
//**************************************************************************

// インクルード宣言
#include <MGLCD.h>                      // 出典；しなぷすのハード製作記　https://synapse.kyoto/lib/MGLCD/page001.html
#include "mcr_gpt_lib.h"
#include "mcr_ad_lib.h"
#include "mcr_adsw_lib.h"

// プロトタイプ宣言
void AGTCallback(timer_callback_args_t __attribute((unused)) * p_args);

// グローバル変数の宣言
FspTimer fsp_timer;                     // 割り込み関係
unsigned long cnt;

// グローバル変数の宣言
MGLCD_AQM1248A_SoftwareSPI MGLCD(MGLCD_SpiPin4( 12, 11, 10, 9),0); // (SCK,MOSI,CS,DI),Wait
mcr_ad ad;
mcr_adsw sw3,sw2,sw1,sw0;
int sw0_cnt,sw1_cnt,sw2_cnt,sw3_cnt;

void setup()
{
   // LCDの初期化
  while( MGLCD.Reset() );               // CS=10pin  RS=9pin  SCLK=13pin  SDI=11pin
  if( strlen("ｱ") != 1 ) MGLCD.SetCodeMode( MGLCD_CODE_UTF8 );  // 半角カナ表示の有効化
  MGLCD.SetVolumeResistor( 28 );        // コントラストの調整 0～63
  MGLCD.ClearScreen();
  pinMode( 0, 1 );                      // バックライトの端子　出力
  digitalWrite( 0, 1);                  // バックライト　点灯

  // AGT 1msごとの割り込み処理の設定 PCLKB=24MHz ∴TIMER_SOURCE_DIV_1(1分周)なら、1/(24e6*1) * 24000 = 1ms  設定は１小さい値である23999を設定する
  fsp_timer.begin(TIMER_MODE_PERIODIC, AGT_TIMER, 1, 23999, 1, (timer_source_div_t)TIMER_SOURCE_DIV_1, AGTCallback);
  IRQManager::getInstance().addPeripheral(IRQ_AGT, (void*)fsp_timer.get_cfg());
  fsp_timer.open(); 

  ad.useCh( 5 );                        // AN005を使用
  ad.start();                           // A/D変換開始

  sw3.begin(  3900,  4100, 500, 50 );   // SW3 ONとみなすAD値の範囲最小、最大、1回目押してからキーリピートするまでの時間[ms]、リピート間隔[ms]
  sw2.begin(  7080,  7280, 500, 50 );   // SW2
  sw1.begin(  9950, 10150, 500, 50 );   // SW1
  sw0.begin( 13050, 13250, 500, 50 );   // SW0
  mcr_adsw::setAdCh( 5 );               // ADスイッチとしてAN005を設定
}

void loop()
{
  static int cnt_b;

  if( sw3.getRepeatSw() ) {
    sw3_cnt++;
    if( sw3_cnt >= 10000 ) sw3_cnt = 0;
  }
  if( sw2.getRepeatSw() ) {
    sw2_cnt++;
    if( sw2_cnt >= 10000 ) sw2_cnt = 0;
  }
  if( sw1.getRepeatSw() ) {
    sw1_cnt++;
    if( sw1_cnt >= 10000 ) sw1_cnt = 0;
  }
  if( sw0.getRepeatSw() ) {
    sw0_cnt++;
    if( sw0_cnt >= 10000 ) sw0_cnt = 0;
  }

  if( cnt != cnt_b ) { // 1msに１回実行
    cnt_b = cnt;

    if( cnt == 100 ) {
      MGLCD.Locate( 0, 0 );
      MGLCD.print( "AD_005 = " );
      MGLCD.print( AD_005 );
      MGLCD.print( "    " );

      MGLCD.Locate( 0, 1 );
      MGLCD.print( " SW3  SW2  SW1  SW0" );

      MGLCD.Locate( 0, 2 );
      MGLCD.print( "   " );
      MGLCD.print( sw3.getSw() );
      MGLCD.print( "    " );
      MGLCD.print( sw2.getSw() );
      MGLCD.print( "    " );
      MGLCD.print( sw1.getSw() );
      MGLCD.print( "    " );
      MGLCD.print( sw0.getSw() );
 
      char lcd_buf[22];
      sprintf( lcd_buf,"%4d %4d %4d %4d", sw3_cnt, sw2_cnt, sw1_cnt, sw0_cnt );
      MGLCD.Locate( 0, 3 );
      MGLCD.print( lcd_buf );
   }

    if( cnt == 200 ) {
      MGLCD.Locate( 0, 4 );
      MGLCD.print( "CN1 = " );
      MGLCD.print( digitalRead(  0 ) );
      MGLCD.print( digitalRead(  1 ) );
      MGLCD.print( digitalRead(  8 ) );
      MGLCD.print( digitalRead(  5 ) );
      MGLCD.print( digitalRead( A4 ) );
      MGLCD.print( digitalRead(  4 ) );
      MGLCD.print( digitalRead( A5 ) );
      MGLCD.print( digitalRead( 13 ) );
    }

    if( cnt == 300 ) {
      MGLCD.Locate( 0, 5 );
      MGLCD.print( "CN2 = " );
      MGLCD.print( digitalRead( A3 ) );
      MGLCD.print( digitalRead(  3 ) );
      MGLCD.print( digitalRead( A2 ) );
      MGLCD.print( digitalRead(  2 ) );
      MGLCD.print( digitalRead( A1 ) );
      MGLCD.print( digitalRead(  6 ) );
      MGLCD.print( digitalRead( A0 ) );
      MGLCD.print( digitalRead(  7 ) );
      cnt = 0;
    }
  }
}

// 割り込み処理　1msごとに実行
void AGTCallback(timer_callback_args_t __attribute((unused)) * p_args)
{
  cnt++;
  mcr_adsw::adsw1msProcess();           // 1msごとに実行する
}

//**********************************************************************
// end of file
//**********************************************************************

/*
改訂内容
Ver.1.00 2024.04.01 作成
Ver.1.01 2024.08.09 sw3～0.begin関数の上限、下限AD値を変更
*/
