//**************************************************************************
// 対象マイコン Renesas Electronics RMC_RA4M1(Rev.2.0)(R7FA4M1AB3CFP9 100pin)
// ﾌｧｲﾙ内容     Basicマイコンカー　サーボセンタ・サーボ角度　確認プログラム
// バージョン   Ver.1.00
// Date         2024.04.01
// Copyright    ジャパンマイコンカーラリー実行委員会
// ライセンス   This software is released under the MIT License.
//              http://opensource.org/licenses/mit-license.php
//**************************************************************************

//======================================
// インクルード
//======================================
#include "mcr_gpt_lib.h"                // GPTを使用するためのライブラリ

//======================================
// シンボル定義
//======================================

// モータ、サーボのPWM関係
#define PWM_CYCLE     47999             //  16ms/(1/HOCO*16) = 0.016 /(1/48e6*16) = 48000
#define SERVO_CENTER  4499              // 1.5ms/(1/HOCO*16) = 0.0015/(1/48e6*16) =  4500
#define HANDLE_STEP   27                // １度あたりの値

//======================================
// グローバル変数の宣言
//======================================
volatile int mode;                   // プログラムのモード
volatile int servo_offset;           // サーボオフセット
volatile int servo_angle;            // サーボ角度

//**********************************************************************
// setup
//**********************************************************************
void setup()
{
  servo_offset = SERVO_CENTER;

  setGPTterminal( 4, 6 );                       // P406(GTIOC1B):サーボPWM
  setGPTterminal( 4, 1 );                       // P401(GTIOC6B):左モータPWM
  setGPTterminal( 4, 3 );                       // P403(GTIOC3A):右モータPWM
  GTIOC1B = SERVO_CENTER;                       // サーボPWMのON幅の設定
  startPWM_GPT1( GTIOCB , DIV16, PWM_CYCLE );   // サーボPWMの周期設定(GPT1 GTIOCBを使用)
  GTIOC6B = 0;                                  // 左モータPWMのON幅の設定
  startPWM_GPT6( GTIOCB , DIV16, PWM_CYCLE );   // 左モータPWMの周期設定(GPT6 GTIOCBを使用) 
  GTIOC3A = 0;                                  // 右モータPWMのON幅の設定
  startPWM_GPT3( GTIOCA , DIV16, PWM_CYCLE );   // 右モータPWMの周期設定(GPT3 GTIOCAを使用)

  // RMC-RA4M1基板上のDIPスイッチ、LED
  pinMode( 13, OUTPUT );
  pinMode( 25, INPUT );
  pinMode( 26, INPUT );
  digitalWrite( 13, 1 );            // D13 LED 消灯

  // CN5 モータドライブ基板
  pinMode( 27 , OUTPUT );           // LED2
  pinMode( 28 , OUTPUT );           // LED3
  //pinMode( 29 , OUTPUT );         // サーボPWMなので設定しない
  //pinMode( 30 , OUTPUT );         // 右モータPWMなので設定しない
  pinMode( 31, OUTPUT );            // 右モータ方向
  //pinMode( 32 , OUTPUT );         // 左モータPWMなので設定しない
  pinMode( 33, OUTPUT );            // 左モータ方向
  pinMode( 34, INPUT );             // プッシュスイッチ
  digitalWrite( 27 , 1 );           // LED2 消灯
  digitalWrite( 28 , 1 );           // LED3 消灯

  // CN7 センサ基板
  pinMode( 35, INPUT );
  pinMode( 36, INPUT );
  pinMode( 37, INPUT );
  pinMode( 38, INPUT );
  pinMode( 39, INPUT );
  pinMode( 40, INPUT );
  pinMode( 41, INPUT );
  pinMode( 42, INPUT );

  Serial.begin( 9600 );             // シリアル通信 速度9600bps

  while ( !Serial ) {
    // シリアルが接続されるまで待つ
  }

  Serial.print( "\033[H" );             // デバッグ用画面クリア
  Serial.print( "\033[2J" );
  Serial.println( "" );
  Serial.println( "==================================================" );
  Serial.println( "　必ずTeraTermなどの通信ソフトを使用してください　" );
  Serial.println( "ArduinoIDEのシリアルモニタでは正しく表示できません" );
  Serial.println( "==================================================" );
}

//**********************************************************************
// loop
//**********************************************************************
void loop()
{
  switch( mode ) {
    case 0:
      Serial.print( "\r\n"
                    "\r\n"
                    "\r\n"
                    "サーボセンタ・サーボ角度　確認プログラム\r\n"
                    "\r\n"
                    "'1' : サーボセンタ調整\r\n"
                    "'2' : サーボ角度確認\r\n"
                    "\r\n"
      );
      mode = 1;
      break;

    case 1:
      if ( Serial.available() ) {       // 受信データがあるか？
        char c = Serial.read();         // 1文字読み込む
        if( c == '1' ) {
          Serial.print( "1\r\n" );
          mode = 11;
        } else if( c == '2' ) {
          Serial.print( "2\r\n" );
          servo_angle = 0;
          mode = 21;
        }
      }
      break;

    case 11:
      Serial.print( "\r\n"
                    "'サーボセンタ調整\r\n"
                    "\r\n"
                    "'Z'キー : サーボセンタ値 +1\r\n"
                    "'X'キー : サーボセンタ値 -1\r\n"
                    "\r\n"
                    "'A'キー : サーボセンタ値+10\r\n"
                    "'S'キー : サーボセンタ値-10\r\n"
                    "\r\n"
                    "Enterキー : メニューに戻ります\r\n"
                    "\r\n"
      );
      Serial.print( servo_offset );
      Serial.print( "  \r" );
      mode = 12;
      break;

    case 12:
      if ( Serial.available() ) {       // 受信データがあるか？
        char c = Serial.read();         // 1文字読み込む
        switch( c ) {
          case 'Z':
          case 'z':
            servo_offset++;
            if( servo_offset > 10000 ) {
              servo_offset = 10000;
            }
            Serial.print( servo_offset );
            Serial.print( "  \r" );
            break;

          case 'A':
          case 'a':
            servo_offset += 10;
            if( servo_offset > 10000 ) {
              servo_offset = 10000;
            }
            Serial.print( servo_offset );
            Serial.print( "  \r" );
            break;

          case 'X':
          case 'x':
            servo_offset--;
            if( servo_offset < 1000 ) {
              servo_offset = 1000;
            }
            Serial.print( servo_offset );
            Serial.print( "  \r" );
            break;

          case 'S':
          case 's':
            servo_offset -= 10;
            if( servo_offset < 1000 ) {
              servo_offset = 1000;
            }
            Serial.print( servo_offset );
            Serial.print( "  \r" );
            break;

          case 0x0d:  // エンター
            mode = 0;
            break;

          default:
            break;
        }
        GTIOC1B = servo_offset;
      }
      break;

    case 21:
      Serial.print( "\r\n"
                    "'サーボ角度確認\r\n"
                    "\r\n"
                    "'Z'キー : サーボ角度 -1\r\n"
                    "'X'キー : サーボ角度 +1\r\n"
                    "\r\n"
                    "'A'キー : サーボ角度-3\r\n"
                    "'S'キー : サーボ角度+3\r\n"
                    "\r\n"
                    "Enterキー : メニューに戻ります\r\n"
                    "\r\n"
      );
      Serial.print( servo_angle );
      Serial.print( "  \r" );
      mode = 22;
      break;

    case 22:
      if ( Serial.available() ) {       // 受信データがあるか？
        char c = Serial.read();         // 1文字読み込む
        switch( c ) {
          case 'X':
          case 'x':
            servo_angle++;
            if( servo_angle > 90 ) {
              servo_angle = 90;
            }
            Serial.print( servo_angle );
            Serial.print( "  \r" );
            break;

          case 'S':
          case 's':
            servo_angle += 3;
            if( servo_angle > 90 ) {
              servo_angle = 90;
            }
            Serial.print( servo_angle );
            Serial.print( "  \r" );
            break;

          case 'Z':
          case 'z':
            servo_angle--;
            if( servo_angle < -90 ) {
              servo_angle = -90;
            }
            Serial.print( servo_angle );
            Serial.print( "  \r" );
            break;

          case 'A':
          case 'a':
            servo_angle -= 3;
            if( servo_angle < -90 ) {
              servo_angle = -90;
            }
            Serial.print( servo_angle );
            Serial.print( "  \r" );
            break;

          case 0x0d:  // エンター
            mode = 0;
            break;

          default:
            break;
        }
        GTIOC1B = servo_offset - servo_angle * HANDLE_STEP;
      }
      break;
  }
}

//**********************************************************************
// end of file
//**********************************************************************

/*
改訂内容
Ver.1.00 2024.04.01 作成
*/
