//**************************************************************************
// 対象マイコン Renesas Electronics RMC_RA4M1(Rev.2.0)(R7FA4M1AB3CFP9 100pin)
// ﾌｧｲﾙ内容     Basicマイコンカー　動作確認プログラム
// バージョン   Ver.1.00
// Date         2024.04.01
// Copyright    ジャパンマイコンカーラリー実行委員会
// ライセンス   This software is released under the MIT License.
//              http://opensource.org/licenses/mit-license.php
//**************************************************************************

//======================================
// インクルード
//======================================
#include "mcr_gpt_lib.h"                // GPTを使用するためのライブラリ

//======================================
// シンボル定義
//======================================
#define PWM_CYCLE     47999             //  16ms/(1/HOCO*16) = 0.016 /(1/48e6*16) = 48000
#define SERVO_CENTER  4500              // 1.5ms/(1/HOCO*16) = 0.0015/(1/48e6*16) =  4500
#define HANDLE_STEP   27                // １度あたりの値

//======================================
// プロトタイプ宣言
//======================================
void AGTCallback( timer_callback_args_t __attribute((unused)) * p_args );
unsigned char sensor_inp( unsigned char mask );
unsigned char dipsw_get( void );
unsigned char pushsw_get( void );
unsigned char startbar_get( void );
void led_out( unsigned char led );
void motor( int accele_l, int accele_r );
void handle( int angle );
unsigned long convertBCD_CharToLong( unsigned char hex );

//======================================
// グローバル変数の宣言
//======================================
volatile int pattern;                   /* マイコンカー動作パターン     */
volatile unsigned long cnt1;            /* タイマ用                    */
volatile int sw,swb,swcnt;

// 割り込み関係
FspTimer fsp_timer;

//**********************************************************************
// setup
//**********************************************************************
void setup()
{
  setGPTterminal( 4, 6 );                       // P406(GTIOC1B):サーボPWM
  setGPTterminal( 4, 1 );                       // P401(GTIOC6B):左モータPWM
  setGPTterminal( 4, 3 );                       // P403(GTIOC3A):右モータPWM
  GTIOC1B = SERVO_CENTER;                       // サーボPWMのON幅の設定
  startPWM_GPT1( GTIOCB , DIV16, PWM_CYCLE );   // サーボPWMの周期設定(GPT1 GTIOCBを使用)
  GTIOC6B = 0;                                  // 左モータPWMのON幅の設定
  startPWM_GPT6( GTIOCB , DIV16, PWM_CYCLE );   // 左モータPWMの周期設定(GPT6 GTIOCBを使用) 
  GTIOC3A = 0;                                  // 右モータPWMのON幅の設定
  startPWM_GPT3( GTIOCA , DIV16, PWM_CYCLE );   // 右モータPWMの周期設定(GPT3 GTIOCAを使用)

  // RMC-RA4M1基板上のDIPスイッチ、LED
  pinMode( 13, OUTPUT );
  pinMode( 25, INPUT );
  pinMode( 26, INPUT );
  digitalWrite( 13, 1 );            // D13 LED 消灯

  // CN5 モータドライブ基板
  pinMode( 27 , OUTPUT );           // LED2
  pinMode( 28 , OUTPUT );           // LED3
  //pinMode( 29 , OUTPUT );         // サーボPWMなので設定しない
  //pinMode( 30 , OUTPUT );         // 右モータPWMなので設定しない
  pinMode( 31, OUTPUT );            // 右モータ方向
  //pinMode( 32 , OUTPUT );         // 左モータPWMなので設定しない
  pinMode( 33, OUTPUT );            // 左モータ方向
  pinMode( 34, INPUT );             // プッシュスイッチ
  led_out( 0 );

  // CN7 センサ基板
  pinMode( 35, INPUT );
  pinMode( 36, INPUT );
  pinMode( 37, INPUT );
  pinMode( 38, INPUT );
  pinMode( 39, INPUT );
  pinMode( 40, INPUT );
  pinMode( 41, INPUT );
  pinMode( 42, INPUT );

    // AGT 1msごとの割り込み処理の設定 PCLKB=24MHz ∴TIMER_SOURCE_DIV_1(1分周)なら、1/(24e6*1) * 24000 = 1ms  設定は１小さい値である23999を設定する
  fsp_timer.begin( TIMER_MODE_PERIODIC, AGT_TIMER, 1, 23999, 1, (timer_source_div_t)TIMER_SOURCE_DIV_1, AGTCallback );
  IRQManager::getInstance().addPeripheral( IRQ_AGT, (void*)fsp_timer.get_cfg() );
  fsp_timer.open();

  Serial.begin( 9600 );             // シリアル通信 速度9600bps

  while ( !Serial ) {
    // シリアルが接続されるまで待つ
  }
  Serial.print( "\033[H" );         // TeraTerm デバッグ用画面クリア
  Serial.print( "\033[2J" );

  Serial.println( "" );
  Serial.println( "==================================================" );
  Serial.println( "　必ずTeraTermなどの通信ソフトを使用してください　" );
  Serial.println( "ArduinoIDEのシリアルモニタでは正しく表示できません" );
  Serial.println( "==================================================" );
}

//**********************************************************************
// loop
//**********************************************************************
void loop()
{
  int c;

  switch( pattern ) {
    case 0:
      Serial.print( "\r\n"
                    "\r\n"
                    "\r\n"
                    "Basicマイコンカー　動作確認プログラム\r\n"
                    "\r\n"
                    "'1' : プッシュスイッチの動作確認\r\n"
                    "'2' : LED2、LED3の動作確認\r\n"
                    "'3' : サーボの動作確認\r\n"
                    "'4' : 左モータの動作確認\r\n"
                    "'5' : 右モータの動作確認\r\n"
                    "'6' : センサ基板の確認\r\n"
                    "\r\n"
                    "1～6の数字を入力してください "
      );
      pattern = 1;
      break;

    case 1:
      if ( Serial.available() ) {       // 受信データがあるか？
        c = Serial.read();         // 1文字読み込む
        if( c >= '1' && c <= '6' ) {
          Serial.print( (char)c );
          Serial.print( "\r\n" );
          Serial.print( "\r\n" );
          pattern = (c - '0') * 10 + 1;
        }
      }
      break;

    case 11:
      /* プッシュスイッチのテスト */
      Serial.print( "モータドライブ基板のプッシュスイッチを押してください。\r\n" );
      Serial.print( "\r\n" );
      Serial.print( "終わったらどれかキーを押してください。\r\n" );
      pattern = 12;
      break;

    case 12:
      if( cnt1 >= 250 ) {
        cnt1 = 0;
        Serial.print( "Push SW = " );
        Serial.print( pushsw_get() );
        Serial.print( "\r" );
      }

      led_out( pushsw_get() * 3 );
      if ( Serial.available() != 0 ) {  // 受信データがあるか？
        c = Serial.read();         // 1文字ダミーリード
        pattern = 0;
      }
      break;
    
    case 21:
      /* LEDのテスト */
      Serial.print( "現在LED2,LED3を点灯中です。\r\n" );
      Serial.print( "\r\n" );
      Serial.print( "終わったらどれかのキーを押してください。\r\n" );
      pattern = 22;
      break;

    case 22:
      led_out( (cnt1 / 500) % 2 + 1 );
      if ( Serial.available() != 0 ) {  // 受信データがあるか？
        c = Serial.read();         // 1文字ダミーリード
        led_out( 0 );
        pattern = 0;
      }
      break;

    case 31:
      /* サーボのテスト */
      Serial.print( "サーボの動作確認をします。\r\n" );
      Serial.print( "スペースキーを押すたびに、０度→右３０度→０度→左３０度と角度を変更します。\r\n" );
      Serial.print( "終わったらエンターキーを押してください。\r\n" );
      Serial.print( "\r\n" );
      Serial.print( "角度：０度　　　　\r" );
      handle( 0 );
      pattern = 32;
      break;

    case 32:
    case 33:
    case 34:
    case 35:
      if ( Serial.available() != 0 ) {  // 受信データがあるか？
        c = Serial.read();
        if( c == 0x0d ) {             // エンターキー
          handle( 0 );
          pattern = 0;
        } else if( c == ' ' ) {       // スペースキー
          pattern++;
          if( pattern == 36 ) {
            pattern = 32;
          }
          if( pattern == 33 ) {
            Serial.print( "角度：右３０度　　　　\r" );
            handle( 30 );
          } else if( pattern == 35 ) {
            Serial.print( "角度：左３０度　　　　\r" );
            handle( -30 );
          } else {
            Serial.print( "角度：０度　　　　\r" );
            handle( 0 );
          }
        }
      }
      break;
      
    case 41:
      /* 左モータのテスト */
      Serial.print( "左モータの動作確認をします。\r\n" );
      Serial.print( "スペースキーを押すたびに、停止→正転５０％→停止→逆転５０％とPWMを変更します。\r\n" );
      Serial.print( "終わったらエンターキーを押してください。\r\n" );
      Serial.print( "\r\n" );
      Serial.print( "左モータPMW：停止　　　　\r" );
      handle( 0 );
      pattern = 42;
      break;

    case 42:
    case 43:
    case 44:
    case 45:
      if ( Serial.available() != 0 ) {  // 受信データがあるか？
        c = Serial.read();
        if( c == 0x0d ) {             // エンターキー
          motor( 0 , 0 );
          pattern = 0;
        } else if( c == ' ' ) {       // スペースキー
          pattern++;
          if( pattern == 46 ) {
            pattern = 42;
          }
          if( pattern == 43 ) {
            Serial.print( "左モータPMW：正転５０％　　　　\r" );
            motor( 50 , 0 );
          } else if( pattern == 45 ) {
            Serial.print( "左モータPMW：逆転５０％　　　　\r" );
            motor( -50 , 0 );
          } else {
            Serial.print( "左モータPMW：停止　　　　\r" );
            motor( 0 , 0 );
          }
        }
      }
      break;

    case 51:
      /* 右モータのテスト */
      Serial.print( "右モータの動作確認をします。\r\n" );
      Serial.print( "スペースキーを押すたびに、停止→正転５０％→停止→逆転５０％とPWMを変更します。\r\n" );
      Serial.print( "終わったらエンターキーを押してください。\r\n" );
      Serial.print( "\r\n" );
      Serial.print( "右モータPMW：停止　　　　\r" );
      handle( 0 );
      pattern = 52;
      break;

    case 52:
    case 53:
    case 54:
    case 55:
      if ( Serial.available() != 0 ) {  // 受信データがあるか？
        c = Serial.read();
        if( c == 0x0d ) {             // エンターキー
          motor( 0 , 0 );
          pattern = 0;
        } else if( c == ' ' ) {       // スペースキー
          pattern++;
          if( pattern == 56 ) {
            pattern = 52;
          }
          if( pattern == 53 ) {
            Serial.print( "右モータPMW：正転５０％　　　　\r" );
            motor( 0 , 50 );
          } else if( pattern == 55 ) {
            Serial.print( "右モータPMW：逆転５０％　　　　\r" );
            motor( 0 , -50 );
          } else {
            Serial.print( "右モータPMW：停止　　　　\r" );
            motor( 0 , 0 );
          }
        }
      }
      break;

    case 61:
      /* センサ基板のテスト */
      Serial.print( "センサ基板の確認を行います。\r\n" );
      Serial.print( "終わったらどれかのキーを押してください。\r\n" );
      Serial.print( "\r\n" );
      pattern = 62;
      break;

    case 62:
      if( cnt1 >= 250 ) {
        cnt1 = 0;
        char str[20];
        sprintf( str, "%08ld(0x%02x)", convertBCD_CharToLong( sensor_inp(0xff) ), sensor_inp(0xff) );
        Serial.print( "デジタルセンサ値：");
        Serial.print( str );
        Serial.print( " スタートバー：");
        Serial.print( startbar_get() );
        Serial.print( "\r" );
      }
      if ( Serial.available() != 0 ) {  // 受信データがあるか？
        c = Serial.read();         // 1文字ダミーリード
        pattern = 0;
      }
      break;
  }
}

//******************************************************************
// 1msごとの割り込み（通常走行、1msごとに実行する処理）
//******************************************************************
void AGTCallback( timer_callback_args_t __attribute((unused)) * p_args )
{
  cnt1++;
}

//******************************************************************
// センサ状態検出
// 引数　 マスク値
// 戻り値 センサ値
//******************************************************************
unsigned char sensor_inp( unsigned char mask )
{
  unsigned char sensor;

  sensor  = (R_PORT0->PIDR_b.PIDR12) << 7;  // D35
  sensor |= (R_PORT0->PIDR_b.PIDR11) << 6;  // D36
  sensor |= (R_PORT0->PIDR_b.PIDR10) << 5;  // D37
  sensor |= (R_PORT0->PIDR_b.PIDR8 ) << 4;  // D38
  sensor |= (R_PORT0->PIDR_b.PIDR7 ) << 3;  // D39
  sensor |= (R_PORT0->PIDR_b.PIDR6 ) << 2;  // D40
  sensor |= (R_PORT0->PIDR_b.PIDR5 ) << 1;  // D41
  sensor |= (R_PORT0->PIDR_b.PIDR4 ) << 0;  // D42

  sensor = (~sensor) & mask;

  return sensor;
}

//******************************************************************
// ディップスイッチ値読み込み
// 戻り値 スイッチ値 0～3
//******************************************************************
unsigned char dipsw_get( void )
{
  unsigned char sw = ( !(R_PORT3->PIDR_b.PIDR6) << 1 ) + ( !(R_PORT3->PIDR_b.PIDR7) );

  return sw;
}

//*******************************************************************
// プッシュスイッチ値読み込み
// 戻り値 スイッチ値 ON:1 OFF:0
//*******************************************************************
unsigned char pushsw_get( void )
{
  unsigned char sw = !(R_PORT0->PIDR_b.PIDR3);

  return sw;
}

//*******************************************************************
// スタートバー検出センサ読み込み
// 戻り値 センサ値 ON(バーあり):1 OFF(なし):0
//*******************************************************************
unsigned char startbar_get( void )
{
  unsigned char b = !(R_PORT0->PIDR_b.PIDR4);

  return b;
}

//*******************************************************************
// LED制御
// 引数　スイッチ値 LED2:bit1 LED3:bit0  "0":消灯 "1":点灯
// 例)0x3→LED2:ON LED3:ON  0x2→LED2:ON LED3:OFF
//*******************************************************************
void led_out( unsigned char led )
{
  R_PORT4->PODR_b.PODR2 = !(led & 0x02);
  R_PORT4->PODR_b.PODR5 = !(led & 0x01);
}

//*******************************************************************
// LED制御(マイコンボードのD13)
// 引数　"0":消灯 "1":点灯
//*******************************************************************
void led_out_m( unsigned char led )
{
  R_PORT1->PODR_b.PODR11 = !led;
}
//*******************************************************************
// モータ速度制御
// 引数　 左モータ:-100～100、右モータ:-100～100
//        0で停止、100で正転100%、-100で逆転100%
// 戻り値 なし
//*******************************************************************
void motor( int accele_l, int accele_r )
{
#if 0
  int sw_data;

  sw_data  = dipsw_get() + 7;         // 7～10
  accele_l = accele_l * sw_data / 10;
  accele_r = accele_r * sw_data / 10;
#endif

  // 左モータ制御
  if( accele_l >= 0 ) {
    R_PORT4->PODR_b.PODR0 = 0;
    accele_l = (long)(PWM_CYCLE + 1) * accele_l / 100;
    GTIOC6B = accele_l;
  } else {
    R_PORT4->PODR_b.PODR0 = 1;
    accele_l = (long)(PWM_CYCLE + 1) * (-accele_l) / 100;
    GTIOC6B = accele_l;
  }

  // 右モータ制御
  if( accele_r >= 0 ) {
    R_PORT4->PODR_b.PODR4 = 0;
    accele_r = (long)(PWM_CYCLE + 1) * accele_r / 100;
    GTIOC3A = accele_r;
  } else {
    R_PORT4->PODR_b.PODR4 = 1;
    accele_r = (long)(PWM_CYCLE + 1) * (-accele_r) / 100;
    GTIOC3A = accele_r;
  }
}

//*******************************************************************
// サーボハンドル操作
// 引数　 サーボ操作角度：-90～90
//        -90で左へ90度、0でまっすぐ、90で右へ90度回転
//*******************************************************************
void handle( int angle )
{
  // サーボが左右逆に動く場合は、「-」を「+」に替えてください
  GTIOC1B = SERVO_CENTER - angle * HANDLE_STEP;
}

//**********************************************************************
// char型データの値をlong型変数に2進数で変換
// 引数　 unsigned char 変換元の8bitデータ
// 戻り値 unsigned long 変換先の変数(0～11111111) ※0か1しかありません
//**********************************************************************
unsigned long convertBCD_CharToLong( unsigned char hex )
{
  unsigned long ret = 0;

  for( int i=0; i<8; i++ ) {
    ret *= 10;
    if( (hex & 0x80) != 0 ) {
      ret += 1;
    }
    hex <<= 1;
  }

  return ret;
}

//**********************************************************************
// end of file
//**********************************************************************

/*
改訂内容
Ver.1.00 2024.04.01 作成
*/
