//**************************************************************************
// ﾌｧｲﾙ内容     AQM1248液晶_SW_10ピンコネクタ_シールド用　電圧出力タクトスイッチのスイッチ値取得
// バージョン   Ver.1.00
// Date         2024.05.18
// Copyright    ジャパンマイコンカーラリー実行委員会
// ライセンス   This software is released under the MIT License.
//              http://opensource.org/licenses/mit-license.php
//**************************************************************************

//======================================
// インクルード
//======================================
#include "Arduino.h"
#include "mcr_adsw_lib.h"

//======================================
// グローバル変数の宣言
//======================================
int mcr_adsw::ad_ch;                    // AD変換　チャンネル番号
int mcr_adsw::ad_buf[4];                // AD値 平均計算用
int mcr_adsw::ad_index;                 // 配列の何個目に入れるか
int mcr_adsw::ad_avg;                   // ４回分の平均値
int mcr_adsw::ad_avg_b;                 // １つ前の平均値

//**********************************************************************
// 初期化プログラム
//**********************************************************************
mcr_adsw::mcr_adsw( uint16_t min, uint16_t max, uint16_t repeat_first, uint16_t repeat )
{
  begin( min, max, repeat_first, repeat );
}

void mcr_adsw::init( uint16_t min, uint16_t max, uint16_t repeat_first, uint16_t repeat )
{
  begin( min, max, repeat_first, repeat );
}

void mcr_adsw::begin( uint16_t min, uint16_t max, uint16_t repeat_first, uint16_t repeat )
{
  ad_min = min;
  ad_max = max;
  repeat_first_time = repeat_first;
  repeat_time = repeat;

  mode = 0;
}

//**********************************************************************
// スイッチ値取得（直値）
//**********************************************************************
int mcr_adsw::getSw( void )
{
  int ret = 0;

  if( ad_avg > (ad_avg_b-100)  && ad_avg < (ad_avg_b+100) ) { // AD値は１つ前と比べて±100以内か
    if( ad_avg >= ad_min && ad_avg <= ad_max ) {              // 最小以上、最大未満か
      ret = 1;
    }
  }

  return ret;
}

//**********************************************************************
// スイッチ値取得（キーリピート機能付き）
//**********************************************************************
int mcr_adsw::getRepeatSw( void )
{
  int sw;
  int ret = 0;

  sw = getSw();

  switch( mode ) {
  case 0:             // 最初に押されたことを検出
    if( sw == 1 ) {
      cnt_sw = millis();
      mode = 1;
      ret = 1;
    }
    break;
  case 1:             // リピートタイム以上経ったか？
    if( millis() - cnt_sw >= 10 ) {  // 10msチャタリング防止
      if( sw == 0 ) {
        mode = 0;
      }
    }
    if( millis() - cnt_sw >= repeat_first_time ) {
      if( sw == 1 ) {
        ret = 1;
        cnt_sw = millis();
        mode = 2;
      }
    }
    break;
  case 2:             // 押されっぱなしか？
    if( sw == 0 ) {
      mode = 0;
    } else if( millis() - cnt_sw >= repeat_time ) {  // 10msチャタリング防止
      cnt_sw = millis();
      ret = 1;
    }
  }
  return ret;  // 1:押されている 0:押されていない
}

//**********************************************************************
// A/D変換の取得チャンネルを設定
//**********************************************************************
void mcr_adsw::setAdCh( int ch )
{
  ad_ch = ch;
}

//**********************************************************************
// 1msごとに実行する処理
//**********************************************************************
void mcr_adsw::adsw1msProcess( void )
{
  static int ad_avg_b1 = 0;
  static int ad_avg_b2 = 0;
  static int ad_avg_b3 = 0;

  int ad = R_ADC0->ADDR_b[ad_ch].ADDR;

  ad_avg_b  = ad_avg_b3;                // 4つ前を更新
  ad_avg_b3 = ad_avg_b2;                // 3つ前を更新
  ad_avg_b2 = ad_avg_b1;                // 2つ前を更新
  ad_avg_b1 = ad_avg;                   // 1つ前を更新

  ad_buf[ad_index++] = ad;
  ad_index &= 3;
  ad_avg = ad_buf[0] + ad_buf[1] + ad_buf[2] + ad_buf[3]; // 4回の合計
  ad_avg >>= 2;                         // ÷４で平均を出す
}

//**********************************************************************
// end of file
//**********************************************************************

/*
改訂内容
Ver.1.00 2024.05.18 作成
*/
