//**************************************************************************
// ﾌｧｲﾙ内容     圧電スピーカー制御ライブラリ
// バージョン   Ver.1.00
// Date         2024.03.28
// Copyright    ジャパンマイコンカーラリー実行委員会
// ライセンス   This software is released under the MIT License.
//              http://opensource.org/licenses/mit-license.php
//**************************************************************************

//======================================
// インクルード
//======================================
#include "Arduino.h"
#include "mcr_sp_lib.h"
#include "mcr_gpt_lib.h"

//**********************************************************************
// 初期化プログラム
//**********************************************************************
mcr_sp::mcr_sp( uint8_t ch, uint8_t ch_ab, uint8_t port1, uint8_t port2, uint16_t f )
{
  begin( ch, ch_ab, port1, port2, f );
}

void mcr_sp::init( uint8_t ch, uint8_t ch_ab, uint8_t port1, uint8_t port2, uint16_t f )
{
  begin( ch, ch_ab, port1, port2, f );
}

void mcr_sp::begin( uint8_t ch, uint8_t ch_ab, uint8_t port1, uint8_t port2, uint16_t f )
{
  setGPTterminal( port1, port2 );       // 圧電スピーカーの端子を指定
  sp_gpt_ch = ch;
  sp_gpt_ab = ch_ab;
  freq = f;
}

//**********************************************************************
// 音階を鳴らす
//**********************************************************************
void mcr_sp::tone_out( int tone )
{
  static int tone_buff = -1;
  int i;

  if( tone == tone_buff ) {
    return;                             // 前回と同じ周波数なら終了
  }

  tone_buff = tone;

  switch( sp_gpt_ch ) { // カウント動作　停止
    case 0: R_GPT0->GTCR_b.CST = 0; break;
    case 1: R_GPT1->GTCR_b.CST = 0; break;
    case 2: R_GPT2->GTCR_b.CST = 0; break;
    case 3: R_GPT3->GTCR_b.CST = 0; break;
    case 4: R_GPT4->GTCR_b.CST = 0; break;
    case 5: R_GPT5->GTCR_b.CST = 0; break;
    case 6: R_GPT6->GTCR_b.CST = 0; break;
    case 7: R_GPT7->GTCR_b.CST = 0; break;
  }

  if( tone != 0 ) {
    i = 12000000 / tone;                // i = tone/(1/(48e6/DIV4))
    switch( sp_gpt_ch ) {
    case 0:
      if( sp_gpt_ab == GTIOCA ) {       // 圧電スピーカーのON幅の設定
        GTIOC0A = tone / 2 - 1;
      } else {
        GTIOC0B = tone / 2 - 1;         // 圧電スピーカーのON幅の設定
      }
      startPWM_GPT0( sp_gpt_ab , DIV4, i - 1 ); // 周期の設定
      break;
    case 1:
      if( sp_gpt_ab == GTIOCA ) {
        GTIOC1A = tone / 2 - 1;
      } else {
        GTIOC1B = tone / 2 - 1;
      }
      startPWM_GPT1( sp_gpt_ab , DIV4, i - 1 );
      break;
    case 2:
      if( sp_gpt_ab == GTIOCA ) {
        GTIOC2A = tone / 2 - 1;
      } else {
        GTIOC2B = tone / 2 - 1;
      }
      startPWM_GPT2( sp_gpt_ab , DIV4, i - 1 );
      break;
    case 3:
      if( sp_gpt_ab == GTIOCA ) {
        GTIOC3A = tone / 2 - 1;
      } else {
        GTIOC3B = tone / 2 - 1;
      }
      startPWM_GPT3( sp_gpt_ab , DIV4, i - 1 );
      break;
    case 4:
      if( sp_gpt_ab == GTIOCA ) {
        GTIOC4A = tone / 2 - 1;
      } else {
        GTIOC4B = tone / 2 - 1;
      }
      startPWM_GPT4( sp_gpt_ab , DIV4, i - 1 );
      break;
    case 5:
      if( sp_gpt_ab == GTIOCA ) {
        GTIOC5A = tone / 2 - 1;
      } else {
        GTIOC5B = tone / 2 - 1;
      }
      startPWM_GPT5( sp_gpt_ab , DIV4, i - 1 );
      break;
    case 6:
      if( sp_gpt_ab == GTIOCA ) {
        GTIOC6A = tone / 2 - 1;
      } else {
        GTIOC6B = tone / 2 - 1;
      }
      startPWM_GPT6( sp_gpt_ab , DIV4, i - 1 );
      break;
    case 7:
      if( sp_gpt_ab == GTIOCA ) {
        GTIOC7A = tone / 2 - 1;
      } else {
        GTIOC7B = tone / 2 - 1;
      }
      startPWM_GPT7( sp_gpt_ab , DIV4, i - 1 );
      break;
    }
  }
}

//**********************************************************************
// 音のパターンを鳴らす
// 1bit=50ms を16bit分鳴らす、0xa000なら、ピッピッと鳴る
//**********************************************************************
void mcr_sp::setSpPattern( uint16_t data )
{
  if( sp_pattern == 0 ) {
    sp_pattern = data;
    cnt_sp = 16 * 50;
  }
}

//**********************************************************************
// setSpPatternを実際に実行する関数、1msごとに実行させる
//**********************************************************************
void mcr_sp::sp1msProcess( void )
{
  if( cnt_sp != 0 ) {
    if( cnt_sp % 50 == 0) {
      if( sp_pattern & 0x8000 ) {
        tone_out( freq );
      } else {
        tone_out( 0 );
      }
      sp_pattern <<= 1;
    }
    cnt_sp--;
    if( cnt_sp == 0 ) {
      tone_out( 0 );
    }
  }
}

//**********************************************************************
// end of file
//**********************************************************************

/*
改訂内容
Ver.1.00 2024.03.28 作成
*/
