//**************************************************************************
// ﾌｧｲﾙ内容     スイッチ値取得（キーリピート機能付き）
// バージョン   Ver.1.00
// Date         2024.03.28
// Copyright    ジャパンマイコンカーラリー実行委員会
// ライセンス   This software is released under the MIT License.
//              http://opensource.org/licenses/mit-license.php
//**************************************************************************

//======================================
// インクルード
//======================================
#include "Arduino.h"
#include "mcr_sw_lib.h"

//**********************************************************************
// 初期化プログラム
//**********************************************************************
mcr_sw::mcr_sw( uint8_t p, uint8_t logic, uint8_t pullup, uint16_t repeat_first, uint16_t repeat )
{
  begin( p, logic, pullup, repeat_first, repeat );
}

void mcr_sw::init( uint8_t p, uint8_t logic, uint8_t pullup, uint16_t repeat_first, uint16_t repeat )
{
  begin( p, logic, pullup, repeat_first, repeat );
}

void mcr_sw::begin( uint8_t p, uint8_t logic, uint8_t pullup, uint16_t repeat_first, uint16_t repeat )
{
  pin = p;
  logic_level = logic; // 0:正論理 1:負論理（反転)
  repeat_first_time = repeat_first;
  repeat_time = repeat;
  
  if( pullup != 0 ) {
    pinMode( pin , INPUT_PULLUP );
  } else {
    pinMode( pin , INPUT );
  }
  mode = 0;
}

//**********************************************************************
// スイッチ値取得（直値）
//**********************************************************************
int mcr_sw::getSw( void )
{
  if( logic_level == 0 ) {
    return( digitalRead( pin ) );
  } else {
    return( !digitalRead( pin ) );
  }
}

//**********************************************************************
// スイッチ値取得（キーリピート機能付き）
//**********************************************************************
int mcr_sw::getRepeatSw( void ) {
  int sw;
  int ret = 0;

  sw = getSw();
  
  switch( mode ) {
  case 0:             // 最初に押されたことを検出
    if( sw == 1 ) {
      cnt_sw = millis();
      mode = 1;
      ret = 1;
    }
    break;
  case 1:             // リピートタイム以上経ったか？
    if( millis() - cnt_sw >= 10 ) {  // 10msチャタリング防止
      if( sw == 0 ) {
        mode = 0;
      }
    }
    if( millis() - cnt_sw >= repeat_first_time ) {
      if( sw == 1 ) {
        ret = 1;
        cnt_sw = millis();
        mode = 2;
      }
    }
    break;
  case 2:             // 押されっぱなしか？
    if( sw == 0 ) {
      mode = 0;
    } else if( millis() - cnt_sw >= repeat_time ) {  // 10msチャタリング防止
      cnt_sw = millis();
      ret = 1;
    }
  }
  return ret;  // 1:押されている 0:押されていない
}

//**********************************************************************
// end of file
//**********************************************************************

/*
改訂内容
Ver.1.00 2024.03.28 作成
*/
