//**************************************************************************
// ﾌｧｲﾙ内容     「rmc_r4m1_advnaced_ver1」モータドライブ基板Advanced・アナログセンサ基板TypeS プログラム
// Copyright    ジャパンマイコンカーラリー実行委員会
// ライセンス   This software is released under the MIT License.
//              http://opensource.org/licenses/mit-license.php
//**************************************************************************

//======================================
// インクルード
//======================================
#include <SPI.h>                        // microSDとSPI通信するためのライブラリ
#include <SD.h>                         // microSD読み書きライブラリ
#include <EEPROM.h>                     // EEP-ROM(データフラッシュメモリ)読み書き
#include "FspTimer.h"                   // fsp_timerを使用するためのライブラリ
#include "mcr_gpt_lib.h"                // GPTを使用するためのライブラリ
#include "mcr_sw_lib.h"                 // スイッチのキーリピートなどを使用するためのライブラリ
#include "mcr_ad_lib.h"                 // A/D変換を使用するためのライブラリ
#include "mcr_sp_lib.h"                 // 圧電スピーカーを使用するためのライブラリ
#include <MGLCD.h>                      // 出典；しなぷすのハード製作記　https://synapse.kyoto/lib/MGLCD/page001.html

//======================================
// シンボル定義
//======================================
const char *C_DATE = __DATE__;          // microSDのファイル日付用 コンパイルした日付
const char *C_TIME = __TIME__;          //                         コンパイルした時間

// ログ処理関係
#define LOG_MAX       (100)             // LOG_MAX × 10ms分のログをRAMに貯める ログが飛ぶときは増やしてください ただし、メモリの上限を超えるとエラーになります
#define LOG_NON       (0)               // ログの状態 何もしない
#define LOG_INIT      (1)               //            初期化
#define LOG_INIT_END  (2)               //            初期化
#define LOG_WRITE     (3)               //            書き込み
#define LOG_END       (4)               //            書き込み終了

#define MOTOR_CYCLE   47999             // 1ms/{1/(HOCO/1)} = 0.001 /{1/(48e6/1)} = 48000
#define FREE          1                 // モータモード　フリー
#define BRAKE         0                 // モータモード　ブレーキ

#define NN 2                            // 音階をNNターブ上げる
#define DO (523*NN)
#define RE (587*NN)
#define MI (659*NN)
#define FA (698*NN)
#define SO (784*NN)
#define RA (880*NN)
#define SHI (988*NN)
#define DO2 (1047*NN)

//======================================
// プロトタイプ宣言
//======================================
void AGTCallback(timer_callback_args_t __attribute((unused)) * p_args);
unsigned char sensor_inp( void );
unsigned char center_inp( void );
unsigned char startbar_get( void );
unsigned char dipsw_get( void );
unsigned char pushsw_get( void );
unsigned char get_cn4( void );
void led_out( unsigned char led );
void led_out_m( unsigned char led );
void lcdBackLight( unsigned char light );
void motor_r( int accele_l, int accele_r );
void motor2_r( int accele_l, int accele_r );
void motor_f( int accele_l, int accele_r );
void motor2_f( int accele_l, int accele_r );
void motor_mode_r( int mode_l, int mode_r );
void motor_mode_f( int mode_l, int mode_r );
void motor_st( int pwm );
int check_crossline( void );
int check_rightline( void );
int check_leftline( void );
int getServoAngle( void );
int getSakaAngle( void );
int getAnalogSensor( void );
void servoControl( void );
void servoControl2( void );
int diff( int pwm );
void readDataFlashParameter( void );
void writeDataFlashParameter( void );
void lcdProcess( void );
unsigned long convertBCD_CharToLong( unsigned char hex );
void set_microSD_dateTime( uint16_t* date, uint16_t* time );

//======================================
// グローバル変数の宣言
//======================================
FspTimer fsp_timer;                     // 割り込み関係
mcr_ad ad;                              // A/D変換用
mcr_sp sp;                              // 圧電スピーカー用
mcr_sw sw4,sw3,sw2,sw1, sw0;            // スイッチSW4～SW0用

volatile int pattern;                   // マイコンカー動作パターン
volatile unsigned long cnt_start;       // スタートしてから1msごとに＋１
volatile unsigned long cnt1;            // タイマ用
volatile unsigned long cnt_check_sen;   // タイマ用
volatile unsigned long cnt_check_enc;   // タイマ用
volatile int f_setup_end;               // setupが終了したら1になる
volatile int angle0_ad;                 // 0度のときのgetServoAngle A/D値

char _buf[128];                         // LCD文字列作業用
char _fn_buf[16];                       // ファイルネーム文字列作業用

// エンコーダ関連
volatile long enc_total;                // 積算値(距離)保存用
volatile long enc_kizyun;               // 基準
volatile int enc;                       // 10ms毎の最新値

//  ステアリングモータ関連
volatile int pwm_trace;                 // 中心線トレーズ時のmotor_st PWM値
volatile int pwm_kakudo;                // 角度制御のときのmotor_st PWM値
volatile int set_angle;                 // 指示角度

// センサ関連
volatile int f_digital_sensor_hosei;    // デジタルセンサ補正 1:する 0:しない
volatile int sensorHoseiPattern;        // センサ状態保持用

// 坂道検出用
volatile unsigned long cnt_up;          // 上り坂 しきい値以上の連続時間
volatile unsigned long cnt_down;        // 下り坂 しきい値以上の連続時間
volatile int saka0_ad;                  // 平地の坂道ボリュームA/D値


// 内輪差値計算用　各マイコンカーに合わせて再計算して下さい
const int revolution_difference[] = {   // 角度から内輪、外輪回転差計算
    100, 98, 97, 95, 94,
    92, 91, 89, 88, 87,
    85, 84, 82, 81, 80,
    78, 77, 75, 74, 73,
    71, 70, 69, 68, 66,
    65, 64, 62, 61, 60,
    58, 57, 56, 54, 53,
    52, 50, 49, 47, 46,
    45, 43, 42, 40, 39,
    37
};

// microSD関連変数
ArduinoSPI SPI(MISO1, MOSI1, SCK1, FORCE_SPI1_MODE); // RMC-RA4M1のmicroSD用SPIを選択(SPIの各端子はpins_arduino.hで定義)
File microSD;                           // microSDのファイルアクセス用
int f_err_sd;                           // 1:mciroSDエラー 0:エラーなし
int log_pattern;                        // ログパターン
int log_st;                             // ログの状態
int log_write;                          // 構造体への書き込み位置
int log_read;                           // 構造体からの読み込み位置
typedef struct {
    uint16_t time;                      // 時間
    uint16_t ptn;                       // パターン
    unsigned char dsensor;              // デジタルセンサ
    int16_t asensor;                    // アナログセンサ
    int16_t angle;                      // 角度
    int16_t encoder;                    // エンコーダ
    char lf;                            // モータ左前
    char rf;                            // モータ右前
    char lr;                            // モータ左後
    char rr;                            // モータ右後
    char st;                            // モータステアリング
    int16_t saka_ad;                    // 坂AD
    char f_updown;                      // 上り坂、下り坂フラグ
} log_t;
log_t log_buff[LOG_MAX];                // ログ保存用構造体
log_t log_raw;                          // ログ現在の値

// DataFlash関連
volatile int df_kyori;                  // 距離
volatile float df_p;                    // PD制御のP値
volatile float df_d;                    // PD制御のD値
volatile int df_p11_enc;                // P11エンコーダ
volatile int df_p11_ang1;               // P11 角度１
volatile uint16_t df_debug;             // デバッグモード

// 液晶関係
MGLCD_AQM1248A_SoftwareSPI MGLCD(MGLCD_SpiPin4( 12, 11, 10, 9),0); // (SCK,MOSI,CS,DI),Wait
volatile unsigned long cnt_lcd;         // LCD処理で使用
volatile int f_lcd;                     // 1msごとに1になる

// 各種センサ
volatile int ang;                       // 角度
volatile int asen;                      // アナログセンサ
volatile int saka;                      // 坂センサ（アナログ）
volatile char f_saka_up;                // 坂アップフラグ 1:UP
volatile char f_saka_down;              // 坂アップフラグ 1:DOWN
volatile float vbat;                    // 電圧

//**********************************************************************
// setup
//**********************************************************************
void setup() {
  // RMC-RA4M1基板上のDIPスイッチ、LED
  pinMode( 25, INPUT );
  pinMode( 26, INPUT );
  pinMode( 13, OUTPUT );
  digitalWrite( 13, 1 );                // D13 LED 消灯

  // モータ
  pinMode( 67, OUTPUT );                // 右前モータ方向
  pinMode( 68, OUTPUT );                // 左前モータ方向
  pinMode( 69, OUTPUT );                // ステアリングモータ方向
  pinMode( 70, OUTPUT );                // 右後モータ方向
  pinMode( 77, OUTPUT );                // 左後モータ方向
  pinMode( 50, OUTPUT );                // 左後モータ フリー・ブレーキ
  pinMode( 51, OUTPUT );                // 右前モータ フリー・ブレーキ
  pinMode( 52, OUTPUT );                // 左前モータ フリー・ブレーキ
  pinMode( 55, OUTPUT );                // 右後モータ フリー・ブレーキ

  setGPTterminal( 6,  3 );                              // P603(GTIOC7A):ステアリングPWM
  setGPTterminal( 6, 10 );                              // P610(GTIOC5B):左前モータPWM
  setGPTterminal( 6,  9 );                              // P609(GTIOC5A):右前モータPWM
  setGPTterminal( 6,  8 );                              // P608(GTIOC4B):右後モータPWM
  setGPTterminal( 1, 15 );                              // P115(GTIOC4A):左後モータPWM
  GTIOC7A = 0;                                          // ステアリングPWMのON幅の設定
  startPWM_GPT7( GTIOCA , DIV1, MOTOR_CYCLE );          // ステアリングPWMの周期設定(GPT7 GTIOCAを使用)
  GTIOC5B = 0;                                          // 左前モータPWMのON幅の設定
  GTIOC5A = 0;                                          // 右前モータPWMのON幅の設定
  startPWM_GPT5( GTIOCA | GTIOCB, DIV1, MOTOR_CYCLE );  // 左右前モータPWMの周期設定(GPT5 GTIOCA,Bを使用)
  GTIOC4B = 0;                                          // 右後モータPWMのON幅の設定
  GTIOC4A = 0;                                          // 右後モータPWMのON幅の設定
  startPWM_GPT4( GTIOCA | GTIOCB, DIV1, MOTOR_CYCLE );  // 左右後モータPWMの周期設定(GPT4 GTIOCA,Bを使用)

  // A/D変換
  ad.useCh( 13 );                       // D39端子 アナログセンサ左
  ad.useCh( 12 );                       // D40端子 アナログセンサ右
  ad.useCh( 23 );                       // D65端子 角度センサ（ポテンショメータ）
  ad.useCh( 18 );                       // D66端子 電圧
  ad.useCh(  8 );                       // D61端子 CN4 5pin　坂センサ（ポテンショメータ）
  ad.start();

  // 圧電スピーカー
  sp.begin( 2 , GTIOCA, 1 , 13 , 4000 ); // GPT2のGTIOCAを使用、端子はP113

  // プッシュスイッチ
  sw4.begin( 45, 1, 1, 1000, 1000 );    // SW4 端子,論理(1=反転),プルアップあり,連続押し2回目の時間,連続の時間
  sw3.begin( 47, 1, 1, 500, 50 );       // SW3
  sw2.begin( 46, 1, 1, 500, 50 );       // SW2
  sw1.begin( 48, 1, 1, 500, 50 );       // SW1
  sw0.begin( 49, 1, 1, 500, 50 );       // SW0

  // センサ基板
  pinMode( 35, INPUT );
  pinMode( 36, INPUT );
  pinMode( 37, INPUT );
  pinMode( 38, INPUT );
  pinMode( 41, INPUT );
  pinMode( 42, INPUT );

  // CN4の入力４端子
  pinMode( 61, INPUT );
  pinMode( 62, INPUT );
  pinMode( 63, INPUT );
  pinMode( 64, INPUT );

  // LED(RGB)
  pinMode(  4, OUTPUT );
  pinMode(  5, OUTPUT );
  pinMode(  6, OUTPUT );
  pinMode(  7, OUTPUT );
  digitalWrite( 6 , 0 );
  led_out( 0 );

  startGPT0_Encoder( 3, 0 , 1, 8 );     // 2相エンコーダ 0A=P300 0B=P108を使用
  //startGPT0_Encoder( 3, 0 , 0, 0 );   // 1相エンコーダ 0A=P300を使用

  // LCDの初期化
  pinMode( 8, OUTPUT );                 // バックライト制御端子
  lcdBackLight( 1 );                    // バックライトON
  while( MGLCD.Reset() );               // MGLCD リセット
  if( strlen("ｱ") != 1 ) MGLCD.SetCodeMode( MGLCD_CODE_UTF8 );  // 半角カナ表示の有効化
  MGLCD.SetVolumeResistor( 28 );        // コントラストの調整 0～63
  MGLCD.ClearScreen();

  // AGT 1msごとの割り込み処理の設定 PCLKB=24MHz ∴TIMER_SOURCE_DIV_1(1分周)なら、1/(24e6*1) * 24000 = 1ms  設定は１小さい値である23999を設定する
  fsp_timer.begin(TIMER_MODE_PERIODIC, AGT_TIMER, 1, 23999, 1, (timer_source_div_t)TIMER_SOURCE_DIV_1, AGTCallback);
  IRQManager::getInstance().addPeripheral(IRQ_AGT, (void*)fsp_timer.get_cfg());
  fsp_timer.open();

  Serial.begin( 9600 );                 // シリアル通信を使用
#if 0
  while( !Serial ) {
    // シリアル接続されるまで待つ
    if( cnt1 >= 3000 ) break;           // タイムアウト
  }
#endif

  Serial.print( "\033[H" );             // デバッグ用画面クリア
  Serial.print( "\033[2J" );

  if( !SD.begin( 2.4e6, CS1 ) ) {       // 通信速度Hz(e6=10の6乗),microSDのCS1(pins_arduino.hで定義)
    f_err_sd = 1;
    Serial.println( "microSDが認識されません!" );       // microSDは認識されず(刺しているのに認識されない場合は2.4e6の値を小さくしてください)
  } else {
    Serial.println( "microSDを認識しました。" );        // microSD認識OK!
    SdFile::dateTimeCallback( &set_microSD_dateTime );  // コールバック関数の定義（日付と時刻を返す関数を登録）
  }

  if( f_err_sd != 0 ) {
    // microSD処理にエラーがあれば音を鳴らし、3秒間LEDの点灯方法を変える
    sp.setSpPattern( 0xcccc );
    cnt1 = 0;
    while( cnt1 < 3000 ) {
      if( cnt1 % 200 < 100 ) {
        led_out( 0x4 );
      } else {
        led_out( 0x0 );
      }
    }
  }

  readDataFlashParameter();             // パラメータ読み込み

  angle0_ad  = AD_023;                  // 0度の位置記憶
  saka0_ad   = AD_008;                  // 平地の坂道ボリュームA/D値記憶

  // マイコンカーの状態初期化
  motor_st( 0 );
  motor_mode_f( BRAKE, BRAKE );
  motor_mode_r( BRAKE, BRAKE );
  motor_f( 0, 0 );
  motor_r( 0, 0 );

  if( df_debug != 0 ) {
    // デバッグモードが１なら音を鳴らし、1.5秒間LEDの点灯方法を変える
    sp.setSpPattern( 0xf0f0 );
    cnt1 = 0;
    while( cnt1 < 1500 ) {
      if( cnt1 % 200 < 100 ) {
        led_out( 0x5 );
      } else {
        led_out( 0x0 );
      }
    }
  }
  sp.setSpPattern( 0x8000 );            // 初期化終了
  f_setup_end = 1;                      // setup完了
}

//**********************************************************************
// loop
//**********************************************************************
void loop() {
  int i;

  // LCD表示処理
  if( f_lcd == 1 ) {
    f_lcd = 0;
    lcdProcess();
  }

  // microSDへのログ保存処理
  if( f_err_sd == 0 ) {
    switch( log_pattern ) {
      case 0:
        // 待機
        if( log_st == LOG_INIT ) {
          log_pattern = 991;
        }
        break;

      case 991:
        // 連番の読み込み
        i = 0;
        Serial.println( "microSDのrenban.txtを読み込みます。" );
        microSD = SD.open( "renban.txt", FILE_READ );
        if( microSD != 0 ) {
          int length = microSD.available();
          if( length > 8 ){
            length = 8;
          }
          microSD.read( _fn_buf, length );
          sscanf( _fn_buf, "%d", &i );
          if( i < 0 || i >= 99999 ) {
              i = 0;
          }
          microSD.close();
        } else {
          Serial.println( "renban.txtが開けませんので「log00001.csv」でファイルを作成します。" );
        }

        // 連番ファイルを一度、削除
        if( SD.exists( "renban.txt" ) ) {
          SD.remove( "renban.txt" );
        }

        // 連番ファイルを新規に作成
        microSD = SD.open( "renban.txt", FILE_WRITE );
        if( microSD != 0 ) {
          sprintf( _fn_buf, "%d", i + 1 );
          microSD.println( _fn_buf );
          microSD.close();
        }

        // 新しい番号でログファイル作成
        sprintf( _fn_buf, "log%05d.csv", i + 1 );
        microSD = SD.open( _fn_buf, FILE_WRITE );
        if( microSD != 0 ) {
          Serial.print( _fn_buf );
          Serial.println( "でログファイルを作成します。" );
          log_pattern = 992;
        } else {
          f_err_sd = 1;
          Serial.println( "ログファイルが作成できません。" );
          log_pattern = 999;
        }
        break;

      case 992:
        // 最初に書き込む内容
        microSD.println( "ms,pattern,D_sen,D_center,A_sen,angle,enc,m_lf,m_rf,m_lr,m_rr,m_sv,saka_ad,f_updown" );
        log_st = LOG_INIT_END;
        log_pattern = 993;
        break;

      case 993:
        if( log_st == LOG_WRITE ) {
          log_pattern = 994;
        }
        break;

      case 994:
        if( log_write != log_read ) {    // 保存するログが更新されたらmicroSDにログを書き込み
          char _log_buf[8];
          sprintf( _log_buf, "%04d", convertBCD_CharToLong( log_buff[log_read].dsensor & 0x0f) );  // センサの値は2進数8桁で表示
          microSD.print( log_buff[log_read].time );
          microSD.print( "," );
          microSD.print( log_buff[log_read].ptn );
          microSD.print( ",'" );
          microSD.print( _log_buf );
          microSD.print( "," );
          microSD.print( (log_buff[log_read].dsensor & 0x10) != 0 ? 1 : 0 );
          microSD.print( "," );
          microSD.print( log_buff[log_read].asensor );
          microSD.print( "," );
          microSD.print( log_buff[log_read].angle );
          microSD.print( "," );
          microSD.print( log_buff[log_read].encoder );
          microSD.print( "," );
          microSD.print( (int)log_buff[log_read].lf );
          microSD.print( "," );
          microSD.print( (int)log_buff[log_read].rf );
          microSD.print( "," );
          microSD.print( (int)log_buff[log_read].lr );
          microSD.print( "," );
          microSD.print( (int)log_buff[log_read].rr );
          microSD.print( "," );
          microSD.print( (int)log_buff[log_read].st );
          microSD.print( "," );
          microSD.print( (int)log_buff[log_read].saka_ad );
          microSD.print( "," );
          microSD.print( (int)log_buff[log_read].f_updown );
          microSD.println( "" );

          log_read++;
          if( log_read >= LOG_MAX ) {
            log_read = 0;
          }
        } else if( log_st != LOG_WRITE ) {  // 最新のログが保存されていて、書き込みでないならログ処理終了
          log_pattern = 995;
        }
        break;

      case 995:
        microSD.close();                // 必ずcloseしないと、ファイルが残らない
        log_pattern = 999;
        break;

      case 999:
        // 何もしない
        break;
    }
  }
}

///*****************************************************************
// 1msごとの割り込み処理（サンプルプログラムの実測で30usで終了）
///*****************************************************************
void AGTCallback(timer_callback_args_t __attribute((unused)) * p_args)
{
  static int enc_buff = 0;              // １回前のエンコーダ値
  int i;
  unsigned char b;

  ang  = getServoAngle();
  saka = getSakaAngle();
  asen = getAnalogSensor();
  vbat = AD_018 * 5.00 / (16383/3);    // 16383:5.00V = AD_018 : 1/3*VBAT

  cnt_start++;
  cnt1++;
  cnt_check_sen++;
  cnt_check_enc++;
  cnt_lcd++;
  f_lcd = 1;

  sp.sp1msProcess();                    // 圧電スピーカー処理

  if( f_setup_end == 0 ) {              // セットアップ実行中はここで終わり
    return;
  }

  servoControl();                       // ステアリングモータ制御（中心線トレース）実測で0.84usで実行
  servoControl2();                      // ステアリングモータ制御（角度）          実測で0.94usで実行

  // 上り坂、下り坂チェック
  f_saka_up   = (saka >= 180);          // 成立なら１、不成立なら０
  f_saka_down = (saka <= -200);

  if( f_saka_up == 1 ) {
    cnt_up++;
  } else {
    cnt_up = 0;
  }
  if( f_saka_down == 1 ) {
    cnt_down++;
  } else {
    cnt_down = 0;
  }

  // 10回中1回実行する処理
  switch( cnt_start % 10 ) {
    case 0:
      if( log_st == LOG_WRITE) {
        log_buff[log_write].time    = cnt_start;
        log_buff[log_write].ptn     = pattern;
        log_buff[log_write].dsensor = sensor_inp() | (center_inp()<<4);
        log_buff[log_write].asensor = asen;
        log_buff[log_write].angle   = ang;
        log_buff[log_write].encoder = enc;
        log_buff[log_write].lf      = log_raw.lf;
        log_buff[log_write].rf      = log_raw.rf;
        log_buff[log_write].lr      = log_raw.lr;
        log_buff[log_write].rr      = log_raw.rr;
        log_buff[log_write].st      = log_raw.st;
        log_buff[log_write].saka_ad = saka;
        log_buff[log_write].f_updown = (f_saka_up<<1) | f_saka_down;
        log_write++;
        if( log_write >= LOG_MAX ) {
          log_write = 0;
        }
      }
      break;
    case 5:
      // エンコーダ制御
      i = INT_GPT0_CNT;
      enc       = i - enc_buff;
      enc_total += enc;
      enc_buff = i;
      break;
  }

  if( pattern >= 11 && pattern <= 1000 ) {
    if( pushsw_get() == 1 ) {           // プッシュスイッチが押されたら停止
      pattern = 1001;
    }

    if( enc_total >= 5459L * df_kyori ) { // 距離による停止
      pattern = 1001;
    }

    if( df_debug == 0 ) {
      b = sensor_inp();                   // 脱輪時の停止処理（デジタルセンサ）
      if( b == 0x0f || b == 0x0d || b == 0x0b ) {
        if( cnt_check_sen >= 300 ) {
          pattern = 1001;
        }
      } else {
        cnt_check_sen = 0;
      }

      if( enc <= 5 ) {                    // 脱輪時の停止処理（ロータリエンコーダ）
        if( cnt_check_enc >= 300 ) {
          pattern = 1001;
        }
      } else {
        cnt_check_enc = 0;
      }
    }
  }


  //////////////// 走行処理 ////////////////
  switch( pattern ) {
    case 00:
#if 0 // ステアリングモータの角度制御実験用
      set_angle = +10 * 48;             // +で左 -で右にハンドルを切ります
      motor_st( pwm_kakudo );
      break;
#endif
      // プッシュスイッチ押下待ち
      motor_st( 0 );
      f_digital_sensor_hosei = 1;       // デジタルセンサ補正 1:あり 0:なし
      if( pushsw_get() == 1 ) {
        sp.setSpPattern( 0x8000 );
        led_out( 0 );
        led_out_m( 0 );
        pattern = 1;
        cnt1 = 0;
        break;
      }
      led_out_m( (cnt1%500) < 3 );      // LED点滅
      if( startbar_get() == 1 ) {
        led_out( (cnt1%200) < 100 );    // スタートバー反応ありなら青色で点滅
      } else {
        if( f_saka_up == 1 ) {          // 坂UP反応なら、警告で黄色表示
          led_out( 0x6 );
        } else if( f_saka_down == 1 ) { // 坂DOWN反応なら、警告で水色表示
          led_out( 0x3 );
        } else {
          led_out( 0 );
        }
      }
      break;

    case 01:
      // プッシュスイッチが離されたら
      if( pushsw_get() == 0 && cnt1 >= 50 ) {
        log_st = LOG_INIT;              // ログ処理開始（ファイルオープンなど)
        writeDataFlashParameter();      // 現在のパラメータ書き込み
        angle0_ad  = AD_023;            // 0度の位置記憶
        saka0_ad = AD_008;              // 平地の坂道ボリュームA/D値記憶
        cnt1 = 0;
        pattern = 3;
        break;
      }
      if( cnt1 >= 500 ) {               // 500ms以上押しっぱなしならパラメータ保存のみ
        sp.setSpPattern( 0xa000 );
        writeDataFlashParameter();
        pattern = 2;
      }
      break;

    case 02:
      // パラメータ保存後、プッシュスイッチが離されたらpattern=0に戻る
      if( pushsw_get() == 0 ) {
        pattern = 0;
      }
      break;

    case 03:
      // microSDの初期化待ち
      if( log_st == LOG_INIT_END ) {    // 初期化が終わったら
        pattern = 4;
      }
      if( f_err_sd != 0 ) {
        pattern = 4;
      }
      break;

    case 04:
      // スタートバー開待ち
      motor_st( pwm_trace / 5 );        // ÷5してブルブルを減らす
      led_out( 0x04 >> (cnt1/50) % 4 ); // スタートバー開待ちは、LEDを高速で点滅させる
      if( startbar_get() == 0 ) {       // スタートバーの反応が無くなったら（開いたら）
        led_out( 0x0 );
        lcdBackLight( df_debug );       // debugモードなら点灯　そうでないなら消灯する
        cnt_start = 0;
        cnt1 = 0;
        cnt_check_sen = 0;
        cnt_check_enc = 0;
        enc_total = 0;
        log_st = LOG_WRITE;             // ログ保存開始
        f_digital_sensor_hosei = 1;     // デジタルセンサ補正ありにする
        pattern = 11;
      }
      break;

    case 11:
      // 通常トレース
      motor_st( pwm_trace );
    
      if( ang >= 1440 ) {               // 約左30度以上なら
        if( enc >= 110 ) {
          motor2_f(   0,   0 );
          motor2_r(   0,   0 );
        } else {
          motor2_f(  50,  80 );
          motor2_r(   0,  80 );
        }
      } else if( ang <= -1440 ) {       // 約右30度以上なら
        if( enc >= 110 ) {
          motor2_f(   0,   0 );
          motor2_r(   0,   0 );
        } else {
          motor2_f(  80,  50 );
          motor2_r(  80,   0 );
        }
      } else if( ang >= df_p11_ang1 ) {  // 初期値500(約左10度) 液晶番号３で調整します
        if( enc >= 138 ) {
          motor2_f(   0,   0 );
          motor2_r(   0,   0 );
        } else if( enc >= 110 ) {
          motor2_f( 50, 80 );
          motor2_r(  0, 80 );
        } else {
          motor2_f( diff(80), 80 );
          motor2_r( diff(80), 80 );
        }
     } else if( ang <= -df_p11_ang1 ) {  // 初期値-500(約右10度) 液晶番号３で調整します
        if( enc >= 138 ) {
          motor2_f(  0,   0 );
          motor2_r(  0,   0 );
        } else if( enc >= 110 ) {
          motor2_f( 80, 50 );
          motor2_r( 80,  0 );
        } else {
          motor2_f( 80, diff(80) );
          motor2_r( 80, diff(80) );
        }
      } else {                          // それ以外なら(±10度以内)
        if( enc >= df_p11_enc ) {       // 直線スピード　液晶番号２で調整します
          motor2_f( 0, 0 );
          motor2_r( 0, 0 );
        } else {
          motor2_f( 100, 100 );
          motor2_r( 100, 100 );
        }
      }
      if( check_crossline() == 1 ) {    // クロスラインチェック
        cnt1 = 0;
        f_digital_sensor_hosei = 0;
        pattern = 21;
        break;
      }
      if( check_rightline() == 1 ) {    // 右ハーフラインチェック
        cnt1 = 0;
        f_digital_sensor_hosei = 0;
        pattern = 51;
        break;
      }
      if( check_leftline() == 1 ) {     // 右ハーフラインチェック
        cnt1 = 0;
        f_digital_sensor_hosei = 0;
        pattern = 61;
        break;
      }

      if( cnt_up >= 20 ) {              // 上り坂チェック
        cnt1 = 0;
        enc_kizyun = enc_total;         // ここを基準とする
        pattern = 101;
        break;
      }
      if( cnt_down >= 20 ) {            // 下り坂チェック
        cnt1 = 0;
        enc_kizyun = enc_total;         // ここを基準とする
        pattern = 201;
        break;
      }
      break;

    case 21:
      // クロスライン通過処理
      motor_st( pwm_trace );
      led_out( 0x1 );
      motor_f( 0, 0 );
      motor_r( 0, 0 );
      if( cnt1 >= 50 ) {
        cnt1 = 0;
        pattern = 22;
      }
      break;

    case 22:
      // クロスライン後のトレース、直角検出処理
      motor_st( pwm_trace );
      if( enc >= 115 ) {
        motor2_f( -80, -80 );
        motor2_r( -80, -80 );
      } else if( enc >= 110 ) {
        motor2_f( 0, 0 );
        motor2_r( 0, 0 );
      } else {
        motor2_f( 70, 70 );
        motor2_r( 70, 70 );
      }

      if( (sensor_inp() & 0x01) == 0x01 ) { // 右クランク？
        led_out( 0x2 );
        cnt1 = 0;
        pattern = 31;
        break;
      }
      if( (sensor_inp() & 0x08) == 0x08 ) {  // 左クランク？
        led_out( 0x4 );
        cnt1 = 0;
        pattern = 41;
        break;
      }
      break;

    case 31:
      // 右クランク処理
      motor_st( 90 );                   // motor_stは、+で右 -で左に回します
      motor_f( 60,  0 );
      motor_r( 49, 22 );
      if( sensor_inp() == 0x04 ) {      // 曲げ終わりチェック
        cnt1 = 0;
        f_digital_sensor_hosei = 1;
        pattern = 32;
      }
      break;

    case 32:
      // 少し時間が経つまで待つ
      motor_st( pwm_trace );
      motor2_r( 80, 80 );
      motor2_f( 80, 80 );
      if( cnt1 >= 100 ) {
        led_out( 0x0 );
        pattern = 11;
      }
      break;

    case 41:
      // 左クランク処理
      motor_st( -90 );                  // motor_stは、+で右 -で左に回します
      motor_f(  0, 60 );
      motor_r( 22, 49 );
      if( sensor_inp() == 0x02 ) {      // 曲げ終わりチェック
        cnt1 = 0;
        f_digital_sensor_hosei = 1;
        pattern = 42;
      }
      break;

    case 42:
      // 少し時間が経つまで待つ
      motor_st( pwm_trace );
      motor2_f( 80, 80 );
      motor2_r( 80, 80 );
      if( cnt1 >= 100 ) {
        led_out( 0x0 );
        pattern = 11;
      }
      break;

    case 51:
      // 右レーンチェンジ　右ハーフライン検出したら
      motor_st( pwm_trace );
      motor2_f( 50, 50 );
      motor2_r( 50, 50 );
      led_out( 0x2 );
      pattern = 52;
      cnt1 = 0;
      break;

    case 52:
      // 右レーンチェンジ　右ハーフラインを通り過ぎるまで待つ
      motor_st( pwm_trace );
      if( cnt1 >= 50 ) {
        pattern = 53;
      }
      if( check_crossline() == 1 ) {    // クロスラインチェック
        cnt1 = 0;
        pattern = 21;
      }
      break;

    case 53:
      // 右レーンチェンジ　中心線がなくなるまで進む
      motor_st( pwm_trace );
      if( enc >= 110 ) {
        motor2_f( 0, 0 );
        motor2_r( 0, 0 );
      } else {
        motor2_f( 70, 70 );
        motor2_r( 70, 70 );
      }
      if( sensor_inp() == 0x00 && center_inp() == 0 ) {
        pattern = 54;
        break;
      }
      break;

    case 54:
      // 右レーンチェンジ　新しい中心線が見つかるまで右に曲げる
      set_angle = -16 * 48;             // +で左 -で右にハンドルを切ります 1度あたり48
      motor_st( pwm_kakudo );
      motor2_f( 50,  0 );
      motor2_r( 50, 20 );
      if( (sensor_inp() & 0x04) == 0x04 ) {
        pattern = 55;
        led_out( 0x00 );
        f_digital_sensor_hosei = 1;
        cnt1 = 0;
        break;
      }
      break;

    case 55:
      // 右レーンチェンジ　新しい中心線をトレース
      motor_st( pwm_trace );
      motor2_f( 50, 50 );
      motor2_r( 20, 50 );
      if( cnt1 >= 100 ) {
        pattern = 11;
        cnt1 = 0;
      }
      break;

    case 61:
      // 左レーンチェンジ　左ハーフライン検出したら
      motor_st( pwm_trace );
      motor2_f( 50, 50 );
      motor2_r( 50, 50 );
      led_out( 0x4 );
      pattern = 62;
      cnt1 = 0;
      break;

    case 62:
      // 左レーンチェンジ　左ハーフラインを通り過ぎるまで待つ
      motor_st( pwm_trace );
      if( cnt1 >= 50 ) {
        pattern = 63;
      }
      if( check_crossline() == 1 ) {    // クロスラインチェック
        cnt1 = 0;
        pattern = 21;
      }
      break;

    case 63:
      // 左レーンチェンジ　中心線がなくなるまで進む
      motor_st( pwm_trace );
      if( enc >= 110 ) {
        motor2_f( 0, 0 );
        motor2_r( 0, 0 );
      } else {
        motor2_f( 70, 70 );
        motor2_r( 70, 70 );
      }
      if( center_inp() == 0 ) {
        pattern = 64;
        break;
      }
      break;

    case 64:
      // 左レーンチェンジ　新しい中心線が見つかるまで右に曲げる
      set_angle = 16 * 48;              // +で左 -で右にハンドルを切ります 1度あたり48
      motor_st( pwm_kakudo );
      motor2_f(  0, 50 );
      motor2_r( 20, 50 );
      if( (sensor_inp() & 0x02) == 0x02 ) {
        pattern = 65;
        led_out( 0x00 );
        f_digital_sensor_hosei = 1;
        cnt1 = 0;
        break;
      }
      break;

    case 65:
      // 左レーンチェンジ　新しい中心線をトレース
      motor_st( pwm_trace );
      motor2_f( 50, 50 );
      motor2_r( 50, 20 );
      if( cnt1 >= 100 ) {
        pattern = 11;
        cnt1 = 0;
      }
      break;

    case 101:
      // 上り坂 最初の0.3m
      motor_st( pwm_trace );
      led_out( 0x6 ); // 黄（赤＋緑）
      if( enc_total - enc_kizyun >= 5459L * 0.3 ) {  // 0.3m進んだら
        pattern = 102;
      }
      if( enc >= 110 ) {
        motor2_f( 0, 0 );
        motor2_r( 0, 0 );
      } else {
        motor2_f( 70, 70 );
        motor2_r( 70, 70 );
      }
      break;

    case 102:
      // 上り坂 上り始めから1.2m
      motor_st( pwm_trace );
      if( enc_total - enc_kizyun >= 5459L * 1.2 ) {  // 1.2m進んだら
        pattern = 103;
      }
      if( enc >= 165 ) {
        motor2_f( 0, 0 );
        motor2_r( 0, 0 );
      } else {
        motor2_f( 70, 70 );
        motor2_r( 70, 70 );
      }
      break;

    case 103:
      // 上り坂 上り終わりチェック
      motor_st( pwm_trace );
      if( cnt_down >= 20 ) {            // センサバーが下がるのがこの時間以上なら
        enc_kizyun = enc_total;         // ここを基準とする
        pattern = 104;
      }
      if( enc >= 55 ) {
        motor2_f( 0, 0 );
        motor2_r( 0, 0 );
      } else {
        motor2_f( 70, 70 );
        motor2_r( 70, 70 );
      }
      break;

    case 104:
      // 上り坂 上り終わり後、少し走らせる
      motor_st( pwm_trace );
      if( enc_total - enc_kizyun >= 5459L * 0.3 ) {  // 0.3m進んだら
        led_out( 0x0 );
        pattern = 11;
        break;
      }
      if( enc >= 110 ) {
        motor2_f( 0, 0 );
        motor2_r( 0, 0 );
      } else {
        motor2_f( 70, 70 );
        motor2_r( 70, 70 );
      }
      break;

     case 201:
      // 下り坂 最初の0.3m
      motor_st( pwm_trace );
      led_out( 0x3 ); // 水色（緑＋青）
      if( enc_total - enc_kizyun >= 5459L * 0.3 ) {  // 0.3m進んだら
        pattern = 202;
      }
      if( enc >= 110 ) {
        motor2_f( 0, 0 );
        motor2_r( 0, 0 );
      } else {
        motor2_f( 70, 70 );
        motor2_r( 70, 70 );
      }
      break;

    case 202:
      // 下り坂 上り始めから1.2m
      motor_st( pwm_trace );
      if( enc_total - enc_kizyun >= 5459L * 1.2 ) {  // 1.2m進んだら
        pattern = 203;
      }
      if( enc >= 165 ) {
        motor2_f( 0, 0 );
        motor2_r( 0, 0 );
      } else {
        motor2_f( 70, 70 );
        motor2_r( 70, 70 );
      }
      break;

    case 203:
      // 下り坂 下り終わりチェック
      motor_st( pwm_trace );
      if( cnt_up >= 20 ) {              // センサバーが上がるのがこの時間以上なら
        enc_kizyun = enc_total;         // ここを基準とする
        pattern = 204;
      }
      if( enc >= 55 ) {
        motor2_f( 0, 0 );
        motor2_r( 0, 0 );
      } else {
        motor2_f( 70, 70 );
        motor2_r( 70, 70 );
      }
      break;

    case 204:
      // 下り坂 上り終わり後、少し走らせる
      motor_st( pwm_trace );
      if( enc_total - enc_kizyun >= 5459L * 0.3 ) {  // 0.3m進んだら
        led_out( 0x0 );
        pattern = 11;
        break;
      }
      if( enc >= 110 ) {
        motor2_f( 0, 0 );
        motor2_r( 0, 0 );
      } else {
        motor2_f( 70, 70 );
        motor2_r( 70, 70 );
      }
      break;

    case 1001:
      // 停止処理
      motor_st( pwm_trace );
      motor2_f( 0, 0 );
      motor2_r( 0, 0 );
      led_out( 0x0 );
      f_digital_sensor_hosei = 0;
      pattern = 1002;
      cnt1 = 0;
      break;

    case 1002:
      motor_st( pwm_trace );
      if( enc <= 5 ) {
        motor_st( 0 );
        pattern = 1003;
        sp.setSpPattern( 0xf0f0 );
        log_st = LOG_END;
        cnt1 = 0;
        break;
      }
      break;

    case 1003:
      // バックライトを点滅させて終了
      if( (cnt1 % 1000) <= 100 ) {
        lcdBackLight( 1 );
      } else {
        lcdBackLight( 0 );
      }
      break;

    default:
      break;
  }

}

///*****************************************************************
// アナログセンサ基板TypeSのデジタルセンサ値読み込み
// 引数　 なし
// 戻り値 左端、左中、右中、右端のデジタルセンサ 0:黒 1:白
///*****************************************************************
unsigned char sensor_inp( void )
{
  unsigned char c;

  c  = (!R_PORT0->PIDR_b.PIDR12) << 3;
  c |= (!R_PORT0->PIDR_b.PIDR11) << 2;
  c |= (!R_PORT0->PIDR_b.PIDR10) << 1;
  c |= (!R_PORT0->PIDR_b.PIDR8 ) << 0;

  return c;
}

///*****************************************************************
// アナログセンサ基板TypeSの中心デジタルセンサ読み込み
// 引数　 なし
// 戻り値 中心デジタルセンサ 0:黒 1:白
///*****************************************************************
unsigned char center_inp( void )
{
  unsigned char sensor = (!R_PORT0->PIDR_b.PIDR5 );

  return sensor;
}

///*****************************************************************
// アナログセンサ基板TypeSのスタートバー検出センサ読み込み
// 引数　 なし
// 戻り値 0:スタートバーなし 1:スタートバーあり
///*****************************************************************
unsigned char startbar_get( void )
{
  unsigned char sensor = (!R_PORT0->PIDR_b.PIDR4 );

    return sensor;
}

///*****************************************************************
// マイコンボード上のディップスイッチ(2bit)値読み込み
// 引数　 なし
// 戻り値 スイッチ値 0～3
///*****************************************************************
unsigned char dipsw_get( void )
{
  unsigned char sw = ( !(R_PORT3->PIDR_b.PIDR6) << 1 ) + ( !(R_PORT3->PIDR_b.PIDR7) );

  return sw;
}

///*****************************************************************
// プッシュスイッチ値読み込み
// 引数　 なし
// 戻り値 スイッチ値 0:OFF 1:ON
///*****************************************************************
unsigned char pushsw_get( void )
{
  unsigned char sw = !(R_PORT7->PIDR_b.PIDR8);

  return sw;
}

///*****************************************************************
// CN4の状態読み込み
// 引数　 なし
// 戻り値 0～15
///*****************************************************************
unsigned char get_cn4( void )
{
  unsigned char c;

  c  = (R_PORT0->PIDR_b.PIDR13) << 3;
  c |= (R_PORT0->PIDR_b.PIDR15) << 2;
  c |= (R_PORT5->PIDR_b.PIDR5)  << 1;
  c |= (R_PORT5->PIDR_b.PIDR4)  << 0;

  return c;
}

///*****************************************************************
// フルカラーLEDの制御(LED基板D2～D7にを取り付けてください)
// 引数　 3個のLED制御 0:OFF 1:ON  bit2:赤 bit1:緑 bit0:青
// 戻り値 なし
///*****************************************************************
void led_out( unsigned char led )
{
  R_PORT1->PODR_b.PODR7 = ( (led & 0x4) != 0 );  // 式が真なら1、偽なら0の値を持つ
  R_PORT1->PODR_b.PODR3 = ( (led & 0x2) != 0 );
  R_PORT1->PODR_b.PODR2 = ( (led & 0x1) != 0 );
}

///*****************************************************************
// マイコン搭載のD13 LEDの制御
// 引数　 0:OFF 1:ON
// 戻り値 なし
///*****************************************************************
void led_out_m( unsigned char led )
{
  R_PORT1->PODR_b.PODR11 = ( led == 0 );  // 式が真なら1、偽なら0の値を持つ
}

///*****************************************************************
// AQMマイコン搭載のD13 LEDの制御
// 引数　 0:OFF 1:ON
// 戻り値 なし
///*****************************************************************
void lcdBackLight( unsigned char light )
{
  R_PORT3->PODR_b.PODR4 = ( light == 1 ); // 式が真なら1、偽なら0の値を持つ
}

///*****************************************************************
// 後輪の速度制御(ディップスイッチの割合でスピードを落とす)
// 引数　 左モータ:-100～100 , 右モータ:-100～100
//        0で停止、100で正転100%、-100で逆転100%
// 戻り値 なし
///*****************************************************************
void motor_r( int accele_l, int accele_r )
{
  int sw_data;

  sw_data  = dipsw_get() + 7;           // ディップスイッチ読み込み
  accele_l = accele_l * sw_data / 10;
  accele_r = accele_r * sw_data / 10;

  motor2_r( accele_l, accele_r );
}

///*****************************************************************
// 後輪の速度制御2 ディップスイッチには関係しないmotor関数
// 引数　 左モータ:-100～100 , 右モータ:-100～100
//        0で停止、100で正転100%、-100で逆転100%
// 戻り値 なし
///*****************************************************************
void motor2_r( int accele_l, int accele_r )
{
  log_raw.lr = accele_l;
  log_raw.rr = accele_r;

  if( df_debug != 0 ) {   // デバッグモードならモータ0%
    accele_l = 0;
    accele_r = 0;
  }

  // 左モータ制御
  if( accele_l > 0 ) {
    R_PORT1->PODR_b.PODR14 = 0;
    accele_l = (long)(MOTOR_CYCLE + 1) * accele_l / 100;
    GTIOC4A = accele_l;
  } else if( accele_l < 0 ) {
    R_PORT1->PODR_b.PODR14 = 1;
    accele_l = (long)(MOTOR_CYCLE + 1) * (-accele_l) / 100;
    GTIOC4A = accele_l;
  } else {  // 0%
    GTIOC4A = 0;
  }

  // 右モータ制御
  if( accele_r > 0 ) {
    R_PORT6->PODR_b.PODR1 = 0;
    accele_r = (long)(MOTOR_CYCLE + 1) * accele_r / 100;
    GTIOC4B = accele_r;
  } else if( accele_r < 0 ) {
    R_PORT6->PODR_b.PODR1 = 1;
    accele_r = (long)(MOTOR_CYCLE + 1) * (-accele_r) / 100;
    GTIOC4B = accele_r;
  } else {  // 0%
    GTIOC4B = 0;
  }
}

///*****************************************************************
// 前輪の速度制御 (ディップスイッチの割合でスピードを落とす)
// 引数　 左モータ:-100～100 , 右モータ:-100～100
//        0で停止、100で正転100%、-100で逆転100%
// 戻り値 なし
///*****************************************************************
void motor_f( int accele_l, int accele_r )
{
  int sw_data;

  sw_data  = dipsw_get() + 7;           // ディップスイッチ読み込み
  accele_l = accele_l * sw_data / 10;
  accele_r = accele_r * sw_data / 10;

  motor2_f( accele_l, accele_r );
}

///*****************************************************************
// ディップスイッチには関係しないmotor関数
// 引数　 左モータ:-100～100 , 右モータ:-100～100
//        0で停止、100で正転100%、-100で逆転100%
// 戻り値 なし
///*****************************************************************
void motor2_f( int accele_l, int accele_r )
{
  log_raw.lf = accele_l;
  log_raw.rf = accele_r;

  if( df_debug != 0 ) {   // デバッグモードならモータ0%
    accele_l = 0;
    accele_r = 0;
  }

  // 左モータ制御
  if( accele_l > 0 ) {
    R_PORT5->PODR_b.PODR0 = 0;
    accele_l = (long)(MOTOR_CYCLE + 1) * accele_l / 100;
    GTIOC5B = accele_l;
  } else if( accele_l < 0 ) {
    R_PORT5->PODR_b.PODR0 = 1;
    accele_l = (long)(MOTOR_CYCLE + 1) * (-accele_l) / 100;
    GTIOC5B = accele_l;
  } else {
    GTIOC5B = 0;
  }

  // 右モータ制御
  if( accele_r > 0 ) {
    R_PORT5->PODR_b.PODR1 = 0;
    accele_r = (long)(MOTOR_CYCLE + 1) * accele_r / 100;
    GTIOC5A = accele_r;
  } else if( accele_r < 0 ) {
    R_PORT5->PODR_b.PODR1 = 1;
    accele_r = (long)(MOTOR_CYCLE + 1) * (-accele_r) / 100;
    GTIOC5A = accele_r;
  } else {
    GTIOC5A = 0;
  }
}

///*****************************************************************
// 後モータ停止動作（ブレーキ、フリー）
// 引数　 左モータ:FREE or BRAKE , 右モータ:FREE or BRAKE
// 戻り値 なし
///*****************************************************************
void motor_mode_r( int mode_l, int mode_r )
{
  if( mode_l != 0 ) {
    R_PORT4->PODR_b.PODR11 = 1;
  } else {
    R_PORT4->PODR_b.PODR11 = 0;
  }
  if( mode_r != 0 ) {
    R_PORT2->PODR_b.PODR6 = 1;
  } else {
    R_PORT2->PODR_b.PODR6 = 0;
  }
}

///*****************************************************************
// 前モータ停止動作（ブレーキ、フリー）
// 引数　 左モータ:FREE or BRAKE , 右モータ:FREE or BRAKE
// 戻り値 なし
///*****************************************************************
void motor_mode_f( int mode_l, int mode_r )
{
  if( mode_l != 0 ) {
    R_PORT4->PODR_b.PODR9 = 1;
  } else {
    R_PORT4->PODR_b.PODR9 = 0;
  }
  if( mode_r != 0 ) {
    R_PORT4->PODR_b.PODR10 = 1;
  } else {
    R_PORT4->PODR_b.PODR14 = 0;
  }
}

///*****************************************************************
// ステアリングモータ制御(旧名称servoPwmOut)
// 引数　 ステアリングモータPWM：-100～100
//        0で停止、100で正転100%、-100で逆転100%
// 戻り値 なし
///*****************************************************************
void motor_st( int pwm )
{
  int i = getServoAngle();

  // ボリューム値により左リミット制御
  if( i >= 2000 && pattern >= 11 ) {
    if( pwm < -10 ) pwm = 0;
  }
  // ボリューム値により右リミット制御
  if( i <= -2000 && pattern >= 11 ) {
    if( pwm >  10 ) pwm = 0;
  }

  if( pwm >  95 ) pwm =  95;            // 上限チェック
  if( pwm < -95 ) pwm = -95;            // 下限チェック

  log_raw.st = pwm;                     // ログ用データの保存

  if( pwm > 0 ) {
    R_PORT6->PODR_b.PODR0 = 0;
    pwm = (long)(MOTOR_CYCLE + 1) * pwm / 100;
    GTIOC7A = pwm;
  } else if( pwm < 0 ) {
    R_PORT6->PODR_b.PODR0 = 1;
    pwm = (long)(MOTOR_CYCLE + 1) * (-pwm) / 100;
    GTIOC7A = pwm;
  } else {
    GTIOC7A = 0;
  }
}

//**********************************************************************
// クロスライン検出処理
// 引数　 なし
// 戻り値 0:クロスラインなし 1:あり
//**********************************************************************
int check_crossline( void )
{
    int ret = 0;

    unsigned char b = sensor_inp();
    if( b==0x0f || b==0x0e || b==0x0d || b==0x0b || b==0x07 ) {
        ret = 1;
    }
    return ret;
}

//**********************************************************************
// 右ハーフライン検出処理
// 引数　 なし
// 戻り値 0:右ハーフラインなし 1:あり
//**********************************************************************
int check_rightline( void )
{
    int ret = 0;

    if( ( center_inp() == 1 ) && ( sensor_inp() == 0x03 ) ) {
        ret = 1;
    }
    return ret;
}

//**********************************************************************
// 左ハーフライン検出処理
// 引数　 なし
// 戻り値 0:左ハーフラインなし 1:あり
//**********************************************************************
int check_leftline( void )
{
    int ret = 0;

    if( ( center_inp() == 1 ) && ( sensor_inp() == 0x0c ) ) {
        ret = 1;
    }
    return ret;
}

//**********************************************************************
// ステアリング角度取得
// 引数　 なし
// 戻り値 ハンドル角度のA/D値
//**********************************************************************
int getServoAngle( void )
{
  return( (int)AD_023 - angle0_ad );
}

//**********************************************************************
// 坂角度取得
// 引数　 なし
// 戻り値 坂角度のA/D値
//**********************************************************************
int getSakaAngle( void )
{
  return ( saka0_ad - AD_008 );         // 坂のA/D値
}

//**********************************************************************
// アナログセンサ値取得
// 引数　 なし
// 戻り値 センサ値
//**********************************************************************
int getAnalogSensor( void )
{
  unsigned char b = sensor_inp();
  int ret = AD_013 - AD_012;            // アナログセンサ情報取得

  if( f_digital_sensor_hosei == 1 ) {
    // デジタルセンサ補正フラグが１なら補正処理
    switch( sensorHoseiPattern ) {
      case 0:
        if( b == 0x04 ) {
          ret = -10000;
          break;
        }
        if( b == 0x02 ) {
          ret = 10000;
          break;
        }
        if( b == 0x0c || b == 0x08 ) {
          ret = -12000;
          sensorHoseiPattern = 1;
          break;
        }
        if( b == 0x03 || b == 0x01 ) {
          ret = 12000;
          sensorHoseiPattern = 2;
          break;
        }
        break;

      case 1:
        // センサ右寄り
        ret = -12000;
        if( b == 0x04 ) {
          sensorHoseiPattern = 0;
        }
        break;

      case 2:
        // センサ左寄り
        ret = 12000;
        if( b == 0x02 ) {
          sensorHoseiPattern = 0;
        }
        break;
    }
  } else {
    // デジタルセンサ補正フラグが０なら補正処理をしない
    sensorHoseiPattern = 0;
  }

  return ret;
}

//**********************************************************************
// ステアリングモータ PD制御
// 引数　 なし
// 戻り値 グローバル変数 pwm_trace に代入
//**********************************************************************
void servoControl( void )
{
    static int asen_before = 0;         // 前回のアナログセンサ値
    int ret;
    float work_p, work_d;

    // ステアリングモータ用PWM値計算
    work_p = df_p * asen;                   // 比例
    work_d = df_d * (asen_before - asen );  // 微分(目安はPの5～10倍)
    ret = (int)(work_p - work_d);

    // PWMの上限、下限の設定
    if( ret >  90 ) {
        ret =  90;
    }
    if( ret < -90 ) {
        ret = -90;
    }
    pwm_trace = ret;

    asen_before = asen;                 // 次回はこの値が1ms前の値となる
}

//**********************************************************************
// ステアリングモータ 角度のPD制御
// 引数　 なし
// 戻り値 グローバル変数 pwm_kakudo に代入
//**********************************************************************
void servoControl2( void )
{
  static int  angle_before;
  float work_p, work_d, kp, kd;

  kp = 0.625;                           // 比例定数
  kd = 3.125;                           // 微分定数(目安はPの5～10倍)

  // ステアリングモータ用PWM値計算
  work_p = kp * ( ang - set_angle );    // 比例
  work_d = kd * ( angle_before - ang ); // 微分
  int ret = work_p - work_d;

  // PWMの上限、下限の設定
  if( ret >  90 ) {
      ret =  90;
  }
  if( ret < -90 ) {
      ret = -90;
  }
  pwm_kakudo = ret;

  angle_before = ang;                   // 次回はこの値が1ms前の値となる
}

//**********************************************************************
// 外輪のPWMから、内輪のPWMを割り出す　ハンドル角度は現在の値を使用
// 引数　 外輪PWM
// 戻り値 内輪PWM
//**********************************************************************
int diff( int pwm )
{
  int i  = getServoAngle() / 48;        // 1度あたりの増分で割る
  if( i <  0 ) {
    i = -i;
  }
  if( i > 45 ) {
    i = 45;
  }

  int ret = revolution_difference[i] * pwm / 100;

  return ret;
}

//**********************************************************************
// DataFlashのパラメータ読み込み
// 引数         なし
// 戻り値       なし
//**********************************************************************
void readDataFlashParameter( void )
{
  const int EEPROM_CHECK = 0x20240501;
  int i;

  EEPROM.get( 0, i );                   // 0番地から値読み込み

  if( i != EEPROM_CHECK ) {
    //もしデータ書き込まれていないなら、初期化
    Serial.print( "初めての使用と判断し、EEP-ROMを初期化します...　" );
    sp.setSpPattern( 0xcccc );
    df_kyori = 50;
    df_p = 0.0040;
    df_d = 0.0200;
    df_p11_enc = 55;
    df_p11_ang1 = 500;
    EEPROM.put(  0 , EEPROM_CHECK );    // 0番地　チェック用データ　書き込み
    writeDataFlashParameter();          // それ以外　書き込み
  } else {
    // 前回書き込んだデータを読み込み
    EEPROM.get(  4 , df_kyori );
    EEPROM.get(  8 , df_p );
    EEPROM.get( 12 , df_d );
    EEPROM.get( 16 , df_p11_enc );
    EEPROM.get( 20 , df_p11_ang1 );
    EEPROM.get( 24 , df_debug );
  }
}

//**********************************************************************
// DataFlashへパラメータ書き込み
// 引数         なし
// 戻り値       なし
//**********************************************************************
void writeDataFlashParameter( void )
{
    EEPROM.put(  4 , df_kyori );
    EEPROM.put(  8 , df_p );
    EEPROM.put( 12 , df_d );
    EEPROM.put( 16 , df_p11_enc );
    EEPROM.put( 20 , df_p11_ang1 );
    EEPROM.put( 24 , df_debug );
}

//**********************************************************************
// LCDとスイッチを使ったパラメータセット処理
// 引数         なし
// 戻り値       なし
//**********************************************************************
void lcdProcess( void )
{
  static int lcd_pattern = 0;
  static int f_lcd_cls = 0;

  int i;

  if( pattern >= 3 ) {      // 走行モードなら
    if( f_lcd_cls == 0 ) {  // １回のみ実行
      f_lcd_cls = 1;
      MGLCD.ClearScreen();
    }
  } else {
    // スイッチ3　メニュー＋１
    if( sw3.getRepeatSw() ) {
      MGLCD.ClearScreen();
      lcd_pattern++;
      if( lcd_pattern == 7 ) lcd_pattern = 1;
    }

    // スイッチ2　メニュー－１
    if( sw2.getRepeatSw() ) {
      MGLCD.ClearScreen();
      lcd_pattern--;
      if( lcd_pattern == 0 ) lcd_pattern = 6;
    }

    // スイッチ1,0 パラメータ値調整
    switch( lcd_pattern ) {
      case 0:
        lcd_pattern = 1;
        break;

      case 1: // 距離設定
        if( sw1.getRepeatSw() ) {
          df_kyori++;
          if( df_kyori > 100 ) df_kyori = 100;
        }
        if( sw0.getRepeatSw() ) {
          df_kyori--;
          if( df_kyori < 1 ) df_kyori = 1;
        }

        sprintf( _buf, "01 ﾃｲｼｷｮﾘ = %3dm", df_kyori );
        MGLCD.Locate( 0, 0 );
        MGLCD.print( _buf );
        break;

      case 2: // パターン１１のエンコーダ値設定
        if( sw1.getRepeatSw() ) {
          df_p11_enc++;
          if( df_p11_enc > 500 ) df_p11_enc = 500;
        }
        if( sw0.getRepeatSw() ) {
          df_p11_enc--;
          if( df_p11_enc < 0 ) df_p11_enc = 0;
        }

        sprintf( _buf, "02 P11 Encoder = %3d", df_p11_enc );
        MGLCD.Locate( 0, 0 );
        MGLCD.print( _buf );
        break;

      case 3: // パターン１１の角度１　設定
        if( sw1.getRepeatSw() ) {
          df_p11_ang1++;
          if( df_p11_ang1 > 3000 ) df_p11_ang1 = 3000;
        }
        if( sw0.getRepeatSw() ) {
          df_p11_ang1--;
          if( df_p11_ang1 < 0 ) df_p11_ang1 = 0;
        }

        sprintf( _buf, "03 P11 Angle 1 = %4d", df_p11_ang1 );
        MGLCD.Locate( 0, 0 );
        MGLCD.print( _buf );
        break;

      case 4: // P値　設定
        if( sw1.getRepeatSw() ) {
          df_p += 0.0001;
          if( df_p > 10 ) df_p = 10;
        }
        if( sw0.getRepeatSw() ) {
          df_p -= 0.0001;
          if( df_p < 0 ) df_p = 0;
        }

        sprintf( _buf, "04 P = %7.4f", df_p );
        MGLCD.Locate( 0, 0 );
        MGLCD.print( _buf );
        break;

      case 5: // D値　設定
        if( sw1.getRepeatSw() ) {
          df_d += 0.0001;
          if( df_d > 10 ) df_d = 10;
        }
        if( sw0.getRepeatSw() ) {
          df_d -= 0.0001;
          if( df_d < 0 ) df_d = 0;
        }

        sprintf( _buf, "05 D = %7.4f", df_d );
        MGLCD.Locate( 0, 0 );
        MGLCD.print( _buf );
        break;

      case 6: // デバッグ画面
        if( sw1.getRepeatSw() ) {
          df_debug = 1;
        }
        if( sw0.getRepeatSw() ) {
          df_debug = 0;
        }

        sprintf( _buf, "99 debug = %d", df_debug );
        MGLCD.Locate( 0, 0 );
        MGLCD.print( _buf );
        sprintf( _buf, "ｱﾅﾛｸﾞL=%5d R=%5d", AD_013, AD_012 );
        MGLCD.Locate( 0, 1 );
        MGLCD.print( _buf );
        break;
    }
  }

  if( pattern <=2 || df_debug == 1 ) {  // 走行前、または走行中でもデバッグモードなら
    if( cnt_lcd >= 250  ) {
      cnt_lcd = 0;
      if( pattern >=3 && df_debug == 1 ) {
        // 走行中でデバッグモードならパターンを表示
        sprintf( _buf, "pattern = %4d", pattern );
        MGLCD.Locate( 0, 0 );
        MGLCD.print( _buf );
      }
      // 2～5行目にマイコンカーの状態表示
      sprintf( _buf, "ｱL-R=%06d ｶｸ=%06d", AD_013 - AD_012, getServoAngle() );
      MGLCD.Locate( 0, 2 );
      MGLCD.print( _buf );
      sprintf( _buf, "Aｾﾝｻ=%06d en=%06ld", asen, enc_total );
      MGLCD.Locate( 0, 3 );
      MGLCD.print( _buf );
      sprintf( _buf, "Dｾﾝｻ=%x%x b=%d ｻｶ=%05d", center_inp(), sensor_inp(), startbar_get(), saka );
      MGLCD.Locate( 0, 4 );
      MGLCD.print( _buf );
      sprintf( _buf, "VBAT=%6.3fV ｻｶUD=%d %d", vbat, f_saka_up, f_saka_down );
      MGLCD.Locate( 0, 5 );
      MGLCD.print( _buf );
    }
  }
}

//**********************************************************************
// char型データの値をlong型変数に2進数で変換
// 引数　 unsigned char 変換元の8bitデータ
// 戻り値 unsigned long 変換先の変数(0～11111111) ※0か1しかありません
//**********************************************************************
unsigned long convertBCD_CharToLong( unsigned char hex )
{
  unsigned long ret = 0;

  for( int i=0; i<8; i++ ) {
    ret *= 10;
    if( (hex & 0x80) != 0 ) {
      ret += 1;
    }
    hex <<= 1;
  }

  return ret;
}

//**********************************************************************
// モジュール名 getCompileYear
// 処理概要     コンパイルした時の年を取得
// 引数　       なし
// 戻り値       年
//**********************************************************************
int getCompileYear( const char *p )
{
  int i = atoi( p + 7 );

  if( i < 1980 || i > 2107 ) {
    i = 2024;
  }

  return i;
}

//**********************************************************************
// モジュール名 getCompileMonth
// 処理概要     コンパイルした時の月を取得
// 引数　       なし
// 戻り値       月
//**********************************************************************
int getCompileMonth( const char *p )
{
  const char monthStr[] = { "JanFebMarAprMayJunJulAugSepOctNovDec" };

  for( int i=0; i<12; i++ ) {
    int r = strncmp( monthStr + i * 3, p, 3 );
    if( r == 0 ) {
      return i + 1;
    }
  }

  return 1;
}

//**********************************************************************
// モジュール名 getCompileDay
// 処理概要     コンパイルした時の日を取得
// 引数　       なし
// 戻り値       日
//**********************************************************************
int getCompileDay( const char *p )
{
  int i = atoi( p + 4 );

  if( i < 1 || i > 31 ) {
    i = 1;
  }

  return i;
}

//**********************************************************************
// モジュール名 getCompileHour
// 処理概要     コンパイルした時の時を取得
// 引数　       なし
// 戻り値       時
//**********************************************************************
int getCompileHour( const char *p )
{
  int i = atoi( p );

  if( i < 0 || i > 23 ) {
    i = 0;
  }

  return i;
}

//**********************************************************************
// モジュール名 getCompilerMinute
// 処理概要     コンパイルした時の分を取得
// 引数　       なし
// 戻り値       分
//**********************************************************************
int getCompilerMinute( const char *p )
{
  int i = atoi( p + 3 );

  if( i < 0 || i > 59 ) {
    i = 0;
  }

  return i;
}

//**********************************************************************
// モジュール名 getCompilerSecond
// 処理概要     コンパイルした時の秒を取得
// 引数　       なし
// 戻り値       秒
//**********************************************************************
int getCompilerSecond( const char *p )
{
  int i = atoi( p + 6 );

  if( i < 0 || i > 59 ) {
    i = 0;
  }

  return i;
}

//**********************************************************************
// microSD.close時に、ファイルの日付・時刻を設定
//**********************************************************************
void set_microSD_dateTime( uint16_t* date, uint16_t* time )
{
  // FAT_DATEマクロでフィールドを埋めて日付を返す
  *date = FAT_DATE( (uint16_t)getCompileYear( C_DATE ),
                    (uint8_t)getCompileMonth( C_DATE ),
                    (uint8_t)getCompileDay( C_DATE ) );

  // FAT_TIMEマクロでフィールドを埋めて時間を返す
  *time = FAT_TIME( getCompileHour( C_TIME ),
                    getCompilerMinute( C_TIME ),
                    getCompilerSecond( C_TIME ) );
}

//**********************************************************************
// end of file
//**********************************************************************

/*
Ver.1.00 2024.07.07 作成
Ver.1.01 2024.08.09 getAnalogSensor関数のデジタルセンサの状態を追加
*/
