//**************************************************************************
// ﾌｧｲﾙ内容     「rmc_r4m1_advnaced_ver1_test」モータドライブ基板Advanced Ver.1 動作確認プログラム
// Copyright    ジャパンマイコンカーラリー実行委員会
// ライセンス   This software is released under the MIT License.
//              http://opensource.org/licenses/mit-license.php
//**************************************************************************

//======================================
// インクルード
//======================================
#include <SPI.h>                        // microSDとSPI通信するためのライブラリ
#include <SD.h>                         // microSD読み書きライブラリ
#include "FspTimer.h"                   // fsp_timerを使用するためのライブラリ
#include "mcr_gpt_lib.h"                // GPTを使用するためのライブラリ
#include "mcr_ad_lib.h"                 // A/D変換を使用するためのライブラリ
#include <MGLCD.h>                      // 出典；しなぷすのハード製作記　https://synapse.kyoto/lib/MGLCD/page001.html

//======================================
// シンボル定義
//======================================
#define MOTOR_CYCLE 47999               // 1ms/{1/(HOCO/1)} = 0.001 /{1/(48e6/1)} = 48000
#define FREE        1                   // モータモード　フリー
#define BRAKE       0                   // モータモード　ブレーキ

#define NN 2                            // 音階をNNターブ上げる
#define DO (523*NN)
#define RE (587*NN)
#define MI (659*NN)
#define FA (698*NN)
#define SO (784*NN)
#define RA (880*NN)
#define SHI (988*NN)
#define DO2 (1047*NN)

//======================================
// プロトタイプ宣言
//======================================
void AGTCallback(timer_callback_args_t __attribute((unused)) * p_args);
unsigned char sensor_inp( void );
unsigned char center_inp( void );
unsigned char startbar_get( void );
unsigned char dipsw_get( void );
unsigned char dipsw_get2( void );
unsigned char pushsw_get( void );
unsigned char getSwNow( void );
unsigned char get_cn4( void );
void led_out( unsigned char led );
void motor_r( int accele_l, int accele_r );
void motor2_r( int accele_l, int accele_r );
void motor_f( int accele_l, int accele_r );
void motor2_f( int accele_l, int accele_r );
void motor_mode_r( int mode_l, int mode_r );
void motor_mode_f( int mode_l, int mode_r );
void motor_st( int pwm );
void tone_out( int tone );

//======================================
// グローバル変数の宣言
//======================================
// グローバル変数宣言
ArduinoSPI SPI(MISO1, MOSI1, SCK1, FORCE_SPI1_MODE); // RMC-RA4M1のmicroSD用SPIを選択(SPIの各端子はpins_arduino.hで定義)
//ArduinoSPI SPI(MISO, MOSI, SCK, FORCE_SPI_MODE); // Arduino UNO R4のSPIを使用(MISO=12,MOSI=11,SCK=13)
File microSD;                           // microSDのファイルアクセス用
FspTimer fsp_timer;                     // 割り込み関係
mcr_ad ad;                              // A/D変換用

int pattern;                            // 動作パターン
int pattern_m;                          // モータテスト用パターン
int pattern_mb;                         // モータテスト用パターン(1つ前)
unsigned long cnt1;                     // タイマ用

MGLCD_AQM1248A_SoftwareSPI MGLCD(MGLCD_SpiPin4( 12, 11, 10, 9),0); // (SCK,MOSI,CS,DI),Wait

//**********************************************************************
// setup
//**********************************************************************
void setup() {
  // RMC-RA4M1基板上のDIPスイッチ、LED
  pinMode( 25, INPUT );
  pinMode( 26, INPUT );
  pinMode( 13, OUTPUT );
  digitalWrite( 13, 1 );                // D13 LED 消灯

  // モータ
  pinMode( 67, OUTPUT );                // 右前モータ方向
  pinMode( 68, OUTPUT );                // 左前モータ方向
  pinMode( 69, OUTPUT );                // ステアリングモータ方向
  pinMode( 70, OUTPUT );                // 右後モータ方向
  pinMode( 77, OUTPUT );                // 左後モータ方向
  pinMode( 50, OUTPUT );                // 左後モータ フリー・ブレーキ
  pinMode( 51, OUTPUT );                // 右前モータ フリー・ブレーキ
  pinMode( 52, OUTPUT );                // 左前モータ フリー・ブレーキ
  pinMode( 55, OUTPUT );                // 右後モータ フリー・ブレーキ

  setGPTterminal( 6,  3 );                              // P603(GTIOC7A):ステアリングPWM
  setGPTterminal( 6, 10 );                              // P610(GTIOC5B):左前モータPWM
  setGPTterminal( 6,  9 );                              // P609(GTIOC5A):右前モータPWM
  setGPTterminal( 6,  8 );                              // P608(GTIOC4B):右後モータPWM
  setGPTterminal( 1, 15 );                              // P115(GTIOC4A):左後モータPWM
  GTIOC7A = 0;                                          // ステアリングPWMのON幅の設定
  startPWM_GPT7( GTIOCA , DIV1, MOTOR_CYCLE );          // ステアリングPWMの周期設定(GPT7 GTIOCAを使用)
  GTIOC5B = 0;                                          // 左前モータPWMのON幅の設定
  GTIOC5A = 0;                                          // 右前モータPWMのON幅の設定
  startPWM_GPT5( GTIOCA | GTIOCB, DIV1, MOTOR_CYCLE );  // 左右前モータPWMの周期設定(GPT5 GTIOCA,Bを使用)
  GTIOC4B = 0;                                          // 右後モータPWMのON幅の設定
  GTIOC4A = 0;                                          // 右後モータPWMのON幅の設定
  startPWM_GPT4( GTIOCA | GTIOCB, DIV1, MOTOR_CYCLE );  // 左右後モータPWMの周期設定(GPT4 GTIOCA,Bを使用)

  // A/D変換
  ad.useCh( 13 );                       // D39端子 アナログセンサ左
  ad.useCh( 12 );                       // D40端子 アナログセンサ右
  ad.useCh( 23 );                       // D65端子 角度センサ（ポテンショメータ）
  ad.useCh( 18 );                       // D66端子 電圧
  ad.start();

  // 圧電スピーカー
  setGPTterminal( 1, 13 );              // P113(GTIOC2A):圧電スピーカー

  // プッシュスイッチ
  pinMode( 45, INPUT_PULLUP );          // SW4
  pinMode( 46, INPUT_PULLUP );          // SW3
  pinMode( 47, INPUT_PULLUP );          // SW2
  pinMode( 48, INPUT_PULLUP );          // SW1
  pinMode( 49, INPUT_PULLUP );          // SW0

  // センサ基板
  pinMode( 35, INPUT );
  pinMode( 36, INPUT );
  pinMode( 37, INPUT );
  pinMode( 38, INPUT );
  pinMode( 41, INPUT );
  pinMode( 42, INPUT );

  // CN4の入力４端子
  pinMode( 61, INPUT );
  pinMode( 62, INPUT );
  pinMode( 63, INPUT );
  pinMode( 64, INPUT );

  // LED(RGB)
  pinMode(  4, OUTPUT );
  pinMode(  5, OUTPUT );
  pinMode(  6, OUTPUT );
  pinMode(  7, OUTPUT );
  digitalWrite( 6 , 0 );
  led_out( 0 );

  // バックライト
  pinMode( 8 , OUTPUT );

  // AGT 1msごとの割り込み処理の設定 PCLKB=24MHz ∴TIMER_SOURCE_DIV_1(1分周)なら、1/(24e6*1) * 24000 = 1ms  設定は１小さい値である23999を設定する
  fsp_timer.begin(TIMER_MODE_PERIODIC, AGT_TIMER, 1, 23999, 1, (timer_source_div_t)TIMER_SOURCE_DIV_1, AGTCallback);
  IRQManager::getInstance().addPeripheral(IRQ_AGT, (void*)fsp_timer.get_cfg());
  fsp_timer.open();
  Serial.begin( 9600 );
  while( !Serial ) {
    // シリアル接続されるまで待つ
  }

  Serial.print( "\033[H" );             // デバッグ用画面クリア
  Serial.print( "\033[2J" );

  if( !SD.begin( 2.4e6, CS1 ) ) {       // 通信速度Hz(e6=10の6乗),microSDのCS1(pins_arduino.hで定義)  Arduino UNO R4は「CS」にする
    Serial.println( "microSDが認識されません!" ); // microSDは認識されず(刺しているのに認識されない場合は2.4e6の値を小さくしてください)
  } else {
    Serial.println( "microSDを認識しました。" );  // microSD認識OK!
  }

  Serial.println( "" );
  Serial.println( "==================================================" );
  Serial.println( "　必ずTeraTermなどの通信ソフトを使用してください　" );
  Serial.println( "ArduinoIDEのシリアルモニタでは正しく表示できません" );
  Serial.println( "==================================================" );

  // マイコンカーの状態初期化
  motor_mode_f( BRAKE, BRAKE );
  motor_mode_r( BRAKE, BRAKE );
  motor_f( 0, 0 );
  motor_r( 0, 0 );
  motor_st( 0 );
}

//**********************************************************************
// loop
//**********************************************************************
void loop() {
  int i, j;
  char c;

  switch( pattern ) {
  case 0:
    // メニュー
    Serial.println( "" );
    Serial.println( "" );
    Serial.println( "Motor Drive PCB Advanced Ver.1 Test Program(RMC-RA4M1(rev.2.0)" );
    Serial.println( "" );
    Serial.println( "1 : LEDのテスト" );
    Serial.println( "2 : スイッチのテスト" );
    Serial.println( "3 : CN4の入力テスト" );
    Serial.println( "4 : 圧電スピーカーのテスト" );
    Serial.println( "5 : エンコーダのテスト" );
    Serial.println( "6 : ボリュームのテスト" );
    Serial.println( "7 : アナログセンサ基板のテスト" );
    Serial.println( "8 : モータのテスト" );
    Serial.println( "9 : 電源電圧の表示テスト" );
    Serial.println( "0 : AQM1248液晶シールド基板の表示テスト" );
    Serial.println( "" );
    Serial.print( "1-9,0の数字を入力してください " );
    pattern = 1;
    break;

  case 1:
    // 数字の入力
    if( Serial.available() ) {
      c = Serial.read();
      if( c >= '0' && c <= '9' ) {
        Serial.print( c );
        Serial.println( "" );
        Serial.println( "" );
        cnt1 = 0;
        if( c == '0' ) {
          pattern = 201;
        } else {
          pattern = (c - 0x30) * 10 + 1;
        }
      }
    }
    break;

  case 11:
    // LEDのテスト
    Serial.println( "フルカラーLEDを順番に点灯中です。" );
    Serial.println( "終わるには、どれかキーを押してください。" );
    pattern = 12;
    break;

  case 12:
    led_out( 1 << ( (cnt1 / 500) % 3) );
    if( Serial.available() ) {
      c = Serial.read();
      led_out( 0 );
      pattern = 0;
    }
    break;

  case 21:
    // スイッチのテスト
    Serial.println( "プッシュスイッチSW4～SW0の値を表示中です。" );
    Serial.println( "終わったらどれかキーを押してください。" );
    Serial.println( "" );
    pattern = 22;
    break;

  case 22:
    if( Serial.available() ) {
      c = Serial.read();
      pattern = 0;
      break;
    }
    if( cnt1 >= 200 ) {
      cnt1 = 0;
      Serial.print( "プッシュSW4～SW0(16進数5bit)の値 : 0x" );
      i = getSwNow();
      Serial.print( i < 0x10 ? "0" : "" );
      Serial.print( i, HEX );
      Serial.print( " \r" );
    }
    break;

  case 31:
    // CN4の入力テスト
    Serial.println( "CN4に入力されている値を表示中です。" );
    Serial.println( "終わったらどれかキーを押してください。" );
    Serial.println( "" );
    pattern = 32;
    break;

  case 32:
    if( Serial.available() ) {
      c = Serial.read();
      pattern = 0;
      break;
    }
    if( cnt1 >= 200 ) {
      cnt1 = 0;
      Serial.print( "CN4の入力値 : 0x" );
      Serial.print( get_cn4(), HEX );
      Serial.print( " \r" );
    }
    break;

  case 41:
    // 圧電スピーカーのテスト
    Serial.println( "圧電スピーカーを鳴らしています。" );
    Serial.println( "終わったらどれかキーを押してください。" );
    pattern = 42;
    break;

  case 42:
    if( Serial.available() ) {
      c = Serial.read();
      pattern = 0;
      tone_out( 0 );
      break;
    }
    switch( cnt1 / 500 ) {
      case 0: tone_out( DO );  break;
      case 1: tone_out( RE );  break;
      case 2: tone_out( MI );  break;
      case 3: tone_out( FA );  break;
      case 4: tone_out( SO );  break;
      case 5: tone_out( RA );  break;
      case 6: tone_out( SHI ); break;
      case 7: tone_out( DO2 ); break;
      case 8: tone_out( 0 );   break;
      default: cnt1 = 0;       break;
    }
    break;

  case 51:
    // エンコーダのテスト
    Serial.println( "エンコーダの入力パルス数を表示します。" );
    Serial.println( "1 : CN5(2相)のエンコーダテスト" );
    Serial.println( "2 : CN5(1相)のエンコーダテスト(パルスは2ピンに接続)" );
    Serial.println( "3 : CN6(2相)のエンコーダテスト" );
    Serial.println( "4 : CN6(1相)のエンコーダテスト(パルスは2ピンに接続)" );
    Serial.println( "0 : 終了" );
    Serial.println( "" );
    Serial.println( "0～4の数字を入力してください " );
    pattern = 52;
    break;

  case 52:
    if( Serial.available() ) {
      c = Serial.read();
      if( c >= '0' && c <= '4' ) {
        switch( c ) {
          case '0':
            pattern = 0;
            break;
          case '1':
            startGPT0_Encoder( 3, 0 , 1, 8 );  // 2相エンコーダ 0A=P300 0B=P108を使用
            GPT0_CNT = 0;
            pattern = 53;
            break;
          case '2':
            startGPT0_Encoder( 3, 0 , 0, 0 );  // 1相エンコーダ 0A=P300を使用
            GPT0_CNT = 0;
            pattern = 54;
            break;
          case '3':
            startGPT1_Encoder( 4, 5 , 4, 6 );  // 2相エンコーダ 1A=P405(D28端子) 1B=P406(D29端子)を使用
            GPT1_CNT = 0;
            pattern = 55;
            break;
            break;
          case '4':
            startGPT1_Encoder( 4, 5 , 0, 0 );  // 1相エンコーダ 0A=P405(D28端子)を使用
            GPT1_CNT = 0;
            pattern = 56;
            break;
        }
        Serial.println( c );
        Serial.println( "" );
        Serial.println( "" );
        Serial.println( "終わったらどれかキーを押してください。" );
        cnt1 = 0;
      }
    }
    break;

  case 53:
    led_out( (R_PORT3->PIDR_b.PIDR0) + (R_PORT1->PIDR_b.PIDR8) * 2 ); // エンコーダの信号をLEDに表示

    if( Serial.available() ) {
      c = Serial.read();
      led_out( 0 );
      pattern = 0;
      break;
    }
    if( cnt1 >= 200 ) {
      cnt1 = 0;
      Serial.print( "CN5のエンコーダのパルス値 : " );
      Serial.print( INT_GPT0_CNT );
      Serial.print( "         \r" );
    }
    break;

  case 54:
    led_out( (R_PORT3->PIDR_b.PIDR0) + (R_PORT1->PIDR_b.PIDR8) * 2 ); // エンコーダの信号をLEDに表示

    if( Serial.available() ) {
      c = Serial.read();
      led_out( 0 );
      pattern = 0;
      break;
    }
    if( cnt1 >= 200 ) {
      cnt1 = 0;
      Serial.print( "CN5のエンコーダのパルス値 : " );
      Serial.print( GPT0_CNT );
      Serial.print( "         \r" );
    }
    break;

  case 55:
    led_out( (R_PORT4->PIDR_b.PIDR5) + (R_PORT4->PIDR_b.PIDR6) * 2 ); // エンコーダの信号をLEDに表示

    if( Serial.available() ) {
      c = Serial.read();
      led_out( 0 );
      pattern = 0;
      break;
    }
    if( cnt1 >= 200 ) {
      cnt1 = 0;
      Serial.print( "CN6のエンコーダのパルス値 : " );
      Serial.print( INT_GPT1_CNT );
      Serial.print( "         \r" );
    }
    break;

  case 56:
    led_out( (R_PORT4->PIDR_b.PIDR5) + (R_PORT4->PIDR_b.PIDR6) * 2 ); // エンコーダの信号をLEDに表示

    if( Serial.available() ) {
      c = Serial.read();
      led_out( 0 );
      pattern = 0;
      break;
    }
    if( cnt1 >= 200 ) {
      cnt1 = 0;
      Serial.print( "CN6のエンコーダのパルス値 : " );
      Serial.print( GPT1_CNT );
      Serial.print( "         \r" );
    }
    break;

  case 61:
    // ボリュームのテスト
    Serial.println( "ボリュームの電圧を表示中です。" );
    Serial.println( "終わったらどれかキーを押してください。" );
    Serial.println( "" );
    pattern = 62;
    break;

  case 62:
    if( Serial.available() ) {
      c = Serial.read();
      pattern = 0;
      break;
    }
    if( cnt1 >= 200 ) {
      cnt1 = 0;
      i =  AD_023;
      j = (long)500 * i / 16383;
      Serial.print( "ボリュームの電圧 : " );
      Serial.print( i );
      Serial.print( " (" );
      Serial.print( (float)j / 100, 2 );
      Serial.print( "V)    \r" );
    }
    break;

  case 71:
    // アナログセンサ基板TypeSのテスト
    Serial.println( "アナログセンサ基板の値を表示中です。" );
    Serial.println( "終わったらどれかキーを押してください。" );
    Serial.println( "" );
    pattern = 72;
    break;

  case 72:
    if( Serial.available() ) {
      c = Serial.read();
      pattern = 0;
      break;
    }
    if( cnt1 >= 200 ) {
      cnt1 = 0;
      Serial.print( "Left=" );
      Serial.print( AD_013 );
      Serial.print( " , Right=" );
      Serial.print( AD_012 );
      Serial.print( " , Digital=" );
      Serial.print( sensor_inp(), HEX );
      Serial.print( " , Center=" );
      Serial.print( center_inp() );
      Serial.print( " , Bar=" );
      Serial.print( startbar_get() );
      Serial.print( "     \r" );
    }
    break;

  case 81:
    // モータ(5個)のテスト
    Serial.println( "" );
    Serial.println( "モータのテストをします。" );
    Serial.println( "1 : 左後ろモータ" );
    Serial.println( "2 : 右後ろモータ" );
    Serial.println( "3 : ステアリングモータ" );
    Serial.println( "4 : 左前モータ" );
    Serial.println( "5 : 右前モータ" );
    Serial.println( "0 : 終了" );
    Serial.println( "" );
    Serial.print( "0-5の数字を入力してください " );
    pattern = 82;
    break;

  case 82:
    if( Serial.available() ) {
      c = Serial.read();
      if( c == '0' ) {
        pattern = 0;
        break;
      } else if( c >= '1' && c <= '5' ) {
        Serial.print( c );
        Serial.println( "" );
        Serial.println( "" );
        cnt1 = 0;
        pattern = (c - 0x31) * 10 + 101;
      }
    }
    break;

  case 101:
    // 左後ろモータのテスト
    Serial.println( "左後ろモータをテストします。" );
    Serial.println( "スペースキーを押して動作を変えます。" );
    Serial.println( "0キーで終了です。" );
    Serial.println( "" );
    cnt1 = 0;
    pattern_m = 0;
    pattern_mb = -1;
    pattern = 102;
    break;

  case 102:
    if( Serial.available() ) {
      c = Serial.read();
      if( c == '0' ) {
          motor2_r( 0, 0 );
          motor_mode_r( BRAKE , BRAKE );
          pattern = 81;
          break;
      } else if( c == ' ' ) {
          cnt1 = 0;
          pattern_m++;
          if( pattern_m >= 5 ) pattern_m = 0;
      }
    }
    switch( pattern_m ) {
      case 0:
        if( pattern_m != pattern_mb ) {
            Serial.print( "停止中                     \r" );
            pattern_mb = pattern_m;
        }
        motor2_r( 0, 0 );
        motor_mode_r( BRAKE , BRAKE );
        break;
      case 1:
        if( pattern_m != pattern_mb ) {
            Serial.print( "正転50%% <-> ブレーキ動作中\r" );
            pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
            motor2_r( 50, 0 );
            motor_mode_r( BRAKE , BRAKE );
        } else if(  cnt1 <= 2000 ) {
            motor2_r( 0, 0 );
            motor_mode_r( BRAKE , BRAKE );
        } else {
            cnt1 = 0;
        }
        break;
      case 2:
        if( pattern_m != pattern_mb ) {
            Serial.print( "逆転50%% <-> ブレーキ動作中\r" );
            pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
            motor2_r( -50, 0 );
            motor_mode_r( BRAKE , BRAKE );
        } else if(  cnt1 <= 2000 ) {
            motor2_r( 0, 0 );
            motor_mode_r( BRAKE , BRAKE );
        } else {
            cnt1 = 0;
        }
        break;
      case 3:
        if( pattern_m != pattern_mb ) {
            Serial.print( "正転50%% <-> フリー　動作中\r" );
            pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
            motor2_r( 50, 0 );
            motor_mode_r( FREE , BRAKE );
        } else if(  cnt1 <= 2000 ) {
            motor2_r( 0, 0 );
            motor_mode_r( FREE , BRAKE );
        } else {
            cnt1 = 0;
        }
        break;
      case 4:
        if( pattern_m != pattern_mb ) {
            Serial.print( "逆転50%% <-> フリー　動作中\r" );
            pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
            motor2_r( -50, 0 );
            motor_mode_r( FREE , BRAKE );
        } else if(  cnt1 <= 2000 ) {
            motor2_r( 0, 0 );
            motor_mode_r( FREE , BRAKE );
        } else {
            cnt1 = 0;
        }
        break;
    }
    break;

  case 111:
    // 右後ろモータのテスト
    Serial.println( "右後ろモータをテストします。" );
    Serial.println( "スペースキーを押して動作を変えます。" );
    Serial.println( "0キーで終了です。" );
    Serial.println( "" );
    cnt1 = 0;
    pattern_m = 0;
    pattern_mb = -1;
    pattern = 112;
    break;

  case 112:
    if( Serial.available() ) {
      c = Serial.read();
      if( c == '0' ) {
        motor2_r( 0, 0 );
        motor_mode_r( BRAKE , BRAKE );
        pattern = 81;
        break;
      } else if( c == ' ' ) {
        cnt1 = 0;
        pattern_m++;
        if( pattern_m >= 5 ) pattern_m = 0;
      }
    }
    switch( pattern_m ) {
      case 0:
        if( pattern_m != pattern_mb ) {
          Serial.print( "停止中                     \r" );
          pattern_mb = pattern_m;
        }
        motor2_r( 0, 0 );
        motor_mode_r( BRAKE , BRAKE );
        break;
      case 1:
        if( pattern_m != pattern_mb ) {
          Serial.print( "正転50%% <-> ブレーキ動作中\r" );
          pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
          motor2_r( 0, 50 );
          motor_mode_r( BRAKE , BRAKE );
        } else if(  cnt1 <= 2000 ) {
          motor2_r( 0, 0 );
          motor_mode_r( BRAKE , BRAKE );
        } else {
          cnt1 = 0;
        }
        break;
      case 2:
        if( pattern_m != pattern_mb ) {
          Serial.print( "逆転50%% <-> ブレーキ動作中\r" );
          pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
          motor2_r( 0, -50 );
          motor_mode_r( BRAKE , BRAKE );
        } else if(  cnt1 <= 2000 ) {
          motor2_r( 0, 0 );
          motor_mode_r( BRAKE , BRAKE );
        } else {
          cnt1 = 0;
        }
        break;
      case 3:
        if( pattern_m != pattern_mb ) {
          Serial.print( "正転50%% <-> フリー　動作中\r" );
          pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
          motor2_r( 0, 50 );
          motor_mode_r( BRAKE , FREE );
        } else if(  cnt1 <= 2000 ) {
          motor2_r( 0, 0 );
          motor_mode_r( BRAKE , FREE );
        } else {
          cnt1 = 0;
        }
        break;
      case 4:
        if( pattern_m != pattern_mb ) {
          Serial.print( "逆転50%% <-> フリー　動作中\r" );
          pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
          motor2_r( 0, -50 );
          motor_mode_r( BRAKE , FREE );
        } else if( cnt1 <= 2000 ) {
          motor2_r( 0, 0 );
          motor_mode_r( BRAKE , FREE );
        } else {
          cnt1 = 0;
        }
        break;
    }
    break;

  case 121:
    // ステアリングモータのテスト
    Serial.println( "ステアリングモータをテストします。" );
    Serial.println( "スペースキーを押して動作を変えます。" );
    Serial.println( "0キーで終了です。" );
    Serial.println( "" );
    cnt1 = 0;
    pattern_m = 0;
    pattern_mb = -1;
    pattern = 122;
    break;

  case 122:
    if( Serial.available() ) {
      c = Serial.read();
      if( c == '0' ) {
        motor_st( 0 );
        pattern = 81;
        break;
      } else if( c == ' ' ) {
        cnt1 = 0;
        pattern_m++;
        if( pattern_m >= 3 ) pattern_m = 0;
      }
    }
    switch( pattern_m ) {
      case 0:
        if( pattern_m != pattern_mb ) {
          Serial.print( "停止中                     \r" );
          pattern_mb = pattern_m;
        }
        motor_st( 0 );
        break;
      case 1:
        if( pattern_m != pattern_mb ) {
          Serial.print( "正転50%% <-> ブレーキ動作中\r" );
          pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
          motor_st( 50 );
        } else if(  cnt1 <= 2000 ) {
          motor_st( 0 );
        } else {
          cnt1 = 0;
        }
        break;
      case 2:
        if( pattern_m != pattern_mb ) {
          Serial.print( "逆転50%% <-> ブレーキ動作中\r" );
          pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
            motor_st( -50 );
        } else if(  cnt1 <= 2000 ) {
            motor_st( 0 );
        } else {
            cnt1 = 0;
        }
        break;
    }
    break;

  case 131:
    // 左前モータのテスト
    Serial.println( "左前モータをテストします。" );
    Serial.println( "スペースキーを押して動作を変えます。" );
    Serial.println( "0キーで終了です。" );
    Serial.println( "" );
    cnt1 = 0;
    pattern_m = 0;
    pattern_mb = -1;
    pattern = 132;
    break;

  case 132:
    if( Serial.available() ) {
      c = Serial.read();
      if( c == '0' ) {
        motor2_f( 0, 0 );
        motor_mode_f( BRAKE , BRAKE );
        pattern = 81;
        break;
      } else if( c == ' ' ) {
        cnt1 = 0;
        pattern_m++;
        if( pattern_m >= 5 ) pattern_m = 0;
      }
    }
    switch( pattern_m ) {
      case 0:
        if( pattern_m != pattern_mb ) {
          Serial.print( "停止中                     \r" );
          pattern_mb = pattern_m;
        }
        motor2_f( 0, 0 );
        motor_mode_f( BRAKE , BRAKE );
        break;
      case 1:
        if( pattern_m != pattern_mb ) {
          Serial.print( "正転50%% <-> ブレーキ動作中\r" );
          pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
          motor2_f( 50, 0 );
          motor_mode_f( BRAKE , BRAKE );
        } else if(  cnt1 <= 2000 ) {
          motor2_f( 0, 0 );
          motor_mode_f( BRAKE , BRAKE );
        } else {
          cnt1 = 0;
        }
        break;
      case 2:
        if( pattern_m != pattern_mb ) {
          Serial.print( "逆転50%% <-> ブレーキ動作中\r" );
          pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
          motor2_f( -50, 0 );
          motor_mode_f( BRAKE , BRAKE );
        } else if(  cnt1 <= 2000 ) {
          motor2_f( 0, 0 );
          motor_mode_f( BRAKE , BRAKE );
        } else {
          cnt1 = 0;
        }
        break;
      case 3:
        if( pattern_m != pattern_mb ) {
          Serial.print( "正転50%% <-> フリー　動作中\r" );
          pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
          motor2_f( 50, 0 );
          motor_mode_f( FREE , BRAKE );
        } else if(  cnt1 <= 2000 ) {
          motor2_f( 0, 0 );
          motor_mode_f( FREE , BRAKE );
        } else {
          cnt1 = 0;
        }
        break;
      case 4:
        if( pattern_m != pattern_mb ) {
          Serial.print( "逆転50%% <-> フリー　動作中\r" );
          pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
          motor2_f( -50, 0 );
          motor_mode_f( FREE , BRAKE );
        } else if(  cnt1 <= 2000 ) {
          motor2_f( 0, 0 );
          motor_mode_f( FREE , BRAKE );
        } else {
          cnt1 = 0;
        }
        break;
    }
    break;

  case 141:
    // 右前モータのテスト
    Serial.println( "右前モータをテストします。" );
    Serial.println( "スペースキーを押して動作を変えます。" );
    Serial.println( "0キーで終了です。" );
    Serial.println( "" );
    cnt1 = 0;
    pattern_m = 0;
    pattern_mb = -1;
    pattern = 142;
    break;

  case 142:
    if( Serial.available() ) {
      c = Serial.read();
      if( c == '0' ) {
        motor2_f( 0, 0 );
        motor_mode_f( BRAKE , BRAKE );
        pattern = 81;
        break;
      } else if( c == ' ' ) {
        cnt1 = 0;
        pattern_m++;
        if( pattern_m >= 5 ) pattern_m = 0;
      }
    }
    switch( pattern_m ) {
      case 0:
        if( pattern_m != pattern_mb ) {
          Serial.print( "停止中                     \r" );
          pattern_mb = pattern_m;
        }
        motor2_f( 0, 0 );
        motor_mode_f( BRAKE , BRAKE );
        break;
      case 1:
        if( pattern_m != pattern_mb ) {
          Serial.print( "正転50%% <-> ブレーキ動作中\r" );
          pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
          motor2_f( 0, 50 );
          motor_mode_f( BRAKE , BRAKE );
        } else if(  cnt1 <= 2000 ) {
          motor2_f( 0, 0 );
          motor_mode_f( BRAKE , BRAKE );
        } else {
          cnt1 = 0;
        }
        break;
      case 2:
        if( pattern_m != pattern_mb ) {
          Serial.print( "逆転50%% <-> ブレーキ動作中\r" );
          pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
          motor2_f( 0, -50 );
          motor_mode_f( BRAKE , BRAKE );
        } else if(  cnt1 <= 2000 ) {
          motor2_f( 0, 0 );
          motor_mode_f( BRAKE , BRAKE );
        } else {
          cnt1 = 0;
        }
        break;
      case 3:
        if( pattern_m != pattern_mb ) {
          Serial.print( "正転50%% <-> フリー　動作中\r" );
          pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
          motor2_f( 0, 50 );
          motor_mode_f( BRAKE , FREE );
        } else if(  cnt1 <= 2000 ) {
          motor2_f( 0, 0 );
          motor_mode_f( BRAKE , FREE );
        } else {
          cnt1 = 0;
        }
        break;
      case 4:
        if( pattern_m != pattern_mb ) {
          Serial.print( "逆転50%% <-> フリー　動作中\r" );
          pattern_mb = pattern_m;
        }
        if( cnt1 <= 1000 ) {
          motor2_f( 0, -50 );
          motor_mode_f( BRAKE , FREE );
        } else if(  cnt1 <= 2000 ) {
          motor2_f( 0, 0 );
          motor_mode_f( BRAKE , FREE );
        } else {
            cnt1 = 0;
        }
        break;
    }
    break;

  case 91:
    // 電源電圧の表示テスト
    Serial.println( "電源電圧を表示中です。" );
    Serial.println( "終わったらどれかキーを押してください。" );
    Serial.println( "" );
    pattern = 92;
    break;

  case 92:
    if( Serial.available() ) {
      c = Serial.read();
      pattern = 0;
      break;
    }
    if( cnt1 >= 200 ) {
      cnt1 = 0;
      float vbat = AD_018 * 5.00 / (16383/3.00); // 16383:5.00V = AD_018 : 1/3*VBAT
      Serial.print( "電源電圧VBAT : " );
      Serial.print( AD_018 );
      Serial.print( " (" );
      Serial.print( vbat, 3 );
      Serial.print( "V)    \r" );
    }
    break;

  case 201:
    while( MGLCD.Reset() );         // CS=10pin  RS=9pin  SCLK=13pin  SDI=11pin
    if( strlen("ｱ") != 1 ) MGLCD.SetCodeMode( MGLCD_CODE_UTF8 );  // 半角カナ表示の有効化
    MGLCD.SetVolumeResistor( 28 );  // コントラストの調整 0～63
    MGLCD.ClearScreen();

    MGLCD.Locate( 0, 0 );                   // 左から0番目 , 上から0番目
    MGLCD.print( "000000000011111111112" ); // 横21文字
    MGLCD.print( "012345678901234567890" );

    for( char i = ':'; i<= '}'; i++ ) {
      MGLCD.print( i );
    }
    MGLCD.print( "ｱｲｳｴｵｶｷｸｹｺｬｭｮﾜﾝｦ" );       // 半角カタカナ表示

    Serial.println( "AQM1248液晶シールドに文字を表示中です。" );
    Serial.println( "終わったらどれかキーを押してください。" );
    Serial.println( "" );
    pattern = 202;
    cnt1 = 0;
    break;

  case 202:
    if( Serial.available() ) {
      c = Serial.read();
      digitalWrite( 8, 0 );   // バックライトOFF
      pattern = 0;
      break;
    }
    if( (cnt1%2000) <= 1000 ) {
      digitalWrite( 8, 1 );   // バックライトON
    } else {
      digitalWrite( 8, 0 );   // バックライトOFF
    }
    break;

  default:
    break;
  }
}

//******************************************************************
// 1msごとの割り込み処理
//******************************************************************
void AGTCallback(timer_callback_args_t __attribute((unused)) * p_args)
{
  cnt1++;
}

//******************************************************************
// アナログセンサ基板TypeSのデジタルセンサ値読み込み
// 引数　 なし
// 戻り値 左端、左中、右中、右端のデジタルセンサ 0:黒 1:白
//******************************************************************
unsigned char sensor_inp( void )
{
  unsigned char c;

  c  = (!R_PORT0->PIDR_b.PIDR12) << 3;
  c |= (!R_PORT0->PIDR_b.PIDR11) << 2;
  c |= (!R_PORT0->PIDR_b.PIDR10) << 1;
  c |= (!R_PORT0->PIDR_b.PIDR8 ) << 0;

  return c;
}

//******************************************************************
// アナログセンサ基板TypeSの中心デジタルセンサ読み込み
// 引数　 なし
// 戻り値 中心デジタルセンサ 0:黒 1:白
//******************************************************************
unsigned char center_inp( void )
{
  unsigned char sensor = (!R_PORT0->PIDR_b.PIDR5 );

  return sensor;
}

//******************************************************************
// アナログセンサ基板TypeSのスタートバー検出センサ読み込み
// 引数　 なし
// 戻り値 0:スタートバーなし 1:スタートバーあり
//******************************************************************
unsigned char startbar_get( void )
{
  unsigned char sensor = (!R_PORT0->PIDR_b.PIDR4 );

    return sensor;
}

//******************************************************************
// マイコンボード上のディップスイッチ(2bit)値読み込み
// 引数　 なし
// 戻り値 スイッチ値 0～3
//******************************************************************
unsigned char dipsw_get( void )
{
  unsigned char sw = ( !(R_PORT3->PIDR_b.PIDR6) << 1 ) + ( !(R_PORT3->PIDR_b.PIDR7) );

  return sw;
}

//******************************************************************
// プッシュスイッチ値読み込み
// 引数　 なし
// 戻り値 スイッチ値 0:OFF 1:ON
//******************************************************************
unsigned char pushsw_get( void )
{
  unsigned char sw = !(R_PORT7->PIDR_b.PIDR8);

  return sw;
}

//******************************************************************
// プッシュスイッチ5個の値読み込み
// 引数　 なし
// 戻り値 bit4:SW4 bit3:SW3 bit2:SW2 bit1:SW1 bit0:SW0
//        スイッチ値 0:OFF 1:ON
//******************************************************************
unsigned char getSwNow( void )
{
  unsigned char sw;

  sw  = !(R_PORT7->PIDR_b.PIDR8)  << 4;
  sw |= !(R_PORT4->PIDR_b.PIDR14) << 3;
  sw |= !(R_PORT4->PIDR_b.PIDR15) << 2;
  sw |= !(R_PORT4->PIDR_b.PIDR13) << 1;
  sw |= !(R_PORT4->PIDR_b.PIDR12) << 0;

  return sw;
}

//******************************************************************
// CN4の状態読み込み
// 引数　 なし
// 戻り値 0～15
//******************************************************************
unsigned char get_cn4( void )
{
  unsigned char c;

  c  = (R_PORT0->PIDR_b.PIDR13) << 3;
  c |= (R_PORT0->PIDR_b.PIDR15) << 2;
  c |= (R_PORT5->PIDR_b.PIDR5)  << 1;
  c |= (R_PORT5->PIDR_b.PIDR4)  << 0;

  return c;
}

//******************************************************************
// フルカラーLEDの制御(LED基板D2～D7にを取り付けてください)
// 引数　 3個のLED制御 0:OFF 1:ON  bit2:赤 bit1:緑 bit0:青
// 戻り値 なし
//******************************************************************
void led_out( unsigned char led )
{
  R_PORT1->PODR_b.PODR7 = ( (led & 0x4) != 0 );  // 式が真なら1、偽なら0の値を持つ
  R_PORT1->PODR_b.PODR3 = ( (led & 0x2) != 0 );
  R_PORT1->PODR_b.PODR2 = ( (led & 0x1) != 0 );
}

//******************************************************************
// 後輪の速度制御(ディップスイッチの割合でスピードを落とす)
// 引数　 左モータ:-100～100 , 右モータ:-100～100
//        0で停止、100で正転100%、-100で逆転100%
// 戻り値 なし
//******************************************************************
void motor_r( int accele_l, int accele_r )
{
  int sw_data;

  sw_data  = dipsw_get() + 7;           // ディップスイッチ読み込み
  accele_l = accele_l * sw_data / 10;
  accele_r = accele_r * sw_data / 10;

  motor2_r( accele_l, accele_r );
}

//******************************************************************
// 後輪の速度制御2 ディップスイッチには関係しないmotor関数
// 引数　 左モータ:-100～100 , 右モータ:-100～100
//        0で停止、100で正転100%、-100で逆転100%
// 戻り値 なし
//******************************************************************
void motor2_r( int accele_l, int accele_r )
{
  // 左モータ制御
  if( accele_l > 0 ) {
    R_PORT1->PODR_b.PODR14 = 0;
    accele_l = (long)(MOTOR_CYCLE + 1) * accele_l / 100;
    GTIOC4A = accele_l;
  } else if( accele_l < 0 ) {
    R_PORT1->PODR_b.PODR14 = 1;
    accele_l = (long)(MOTOR_CYCLE + 1) * (-accele_l) / 100;
    GTIOC4A = accele_l;
  } else {  // 0%
    GTIOC4A = 0;
  }

  // 右モータ制御
  if( accele_r > 0 ) {
    R_PORT6->PODR_b.PODR1 = 0;
    accele_r = (long)(MOTOR_CYCLE + 1) * accele_r / 100;
    GTIOC4B = accele_r;
  } else if( accele_r < 0 ) {
    R_PORT6->PODR_b.PODR1 = 1;
    accele_r = (long)(MOTOR_CYCLE + 1) * (-accele_r) / 100;
    GTIOC4B = accele_r;
  } else {  // 0%
    GTIOC4B = 0;
  }
}

//******************************************************************
// 前輪の速度制御 (ディップスイッチの割合でスピードを落とす)
// 引数　 左モータ:-100～100 , 右モータ:-100～100
//        0で停止、100で正転100%、-100で逆転100%
// 戻り値 なし 
//******************************************************************
void motor_f( int accele_l, int accele_r )
{
  int sw_data;

  sw_data  = dipsw_get() + 7;           // ディップスイッチ読み込み
  accele_l = accele_l * sw_data / 10;
  accele_r = accele_r * sw_data / 10;

  motor2_f( accele_l, accele_r );
}

//******************************************************************
// ディップスイッチには関係しないmotor関数
// 引数　 左モータ:-100～100 , 右モータ:-100～100
//        0で停止、100で正転100%、-100で逆転100%
// 戻り値 なし
//******************************************************************
void motor2_f( int accele_l, int accele_r )
{
  // 左モータ制御
  if( accele_l > 0 ) {
    R_PORT5->PODR_b.PODR0 = 0;
    accele_l = (long)(MOTOR_CYCLE + 1) * accele_l / 100;
    GTIOC5B = accele_l;
  } else if( accele_l < 0 ) {
    R_PORT5->PODR_b.PODR0 = 1;
    accele_l = (long)(MOTOR_CYCLE + 1) * (-accele_l) / 100;
    GTIOC5B = accele_l;
  } else {
    GTIOC5B = 0;
  }

  // 右モータ制御
  if( accele_r > 0 ) {
    R_PORT5->PODR_b.PODR1 = 0;
    accele_r = (long)(MOTOR_CYCLE + 1) * accele_r / 100;
    GTIOC5A = accele_r;
  } else if( accele_r < 0 ) {
    R_PORT5->PODR_b.PODR1 = 1;
    accele_r = (long)(MOTOR_CYCLE + 1) * (-accele_r) / 100;
    GTIOC5A = accele_r;
  } else {
    GTIOC5A = 0;
  }
}

//******************************************************************
// 後モータ停止動作（ブレーキ、フリー）
// 引数　 左モータ:FREE or BRAKE , 右モータ:FREE or BRAKE
// 戻り値 なし
//******************************************************************
void motor_mode_r( int mode_l, int mode_r )
{
  if( mode_l ) {
    R_PORT4->PODR_b.PODR11 = 1;
  } else {
    R_PORT4->PODR_b.PODR11 = 0;
  }
  if( mode_r ) {
    R_PORT2->PODR_b.PODR6 = 1;
  } else {
    R_PORT2->PODR_b.PODR6 = 0;
  }
}

//******************************************************************
// 前モータ停止動作（ブレーキ、フリー）
// 引数　 左モータ:FREE or BRAKE , 右モータ:FREE or BRAKE
// 戻り値 なし
//******************************************************************
void motor_mode_f( int mode_l, int mode_r )
{
  if( mode_l ) {
    R_PORT4->PODR_b.PODR9 = 1;
  } else {
    R_PORT4->PODR_b.PODR9 = 0;
  }
  if( mode_r ) {
    R_PORT4->PODR_b.PODR10 = 1;
  } else {
    R_PORT4->PODR_b.PODR14 = 0;
  }
}

//******************************************************************
// ステアリングモータ制御
// 引数　 ステアリングモータPWM：-100～100
//        0で停止、100で正転100%、-100で逆転100%
// 戻り値 なし
//******************************************************************
void motor_st( int pwm )
{
  if( pwm > 0 ) {
    R_PORT6->PODR_b.PODR0 = 0;
    pwm = (long)(MOTOR_CYCLE + 1) * pwm / 100;
    GTIOC7A = pwm;
  } else if( pwm < 0 ) {
    R_PORT6->PODR_b.PODR0 = 1;
    pwm = (long)(MOTOR_CYCLE + 1) * (-pwm) / 100;
    GTIOC7A = pwm;
  } else {
    GTIOC7A = 0;
  }
}

//******************************************************************
// 圧電スピーカー制御(テストプログラム版)
// 引数　 周波数[Hz]
// 戻り値 なし
//******************************************************************
void tone_out( int tone )
{
  static int tone_buff = -1;
  int i;

  if( tone == tone_buff ) {
    return;                             // 前回と同じ周波数なら終了
  }

  tone_buff = tone;
  R_GPT2->GTCR_b.CST = 0;               // カウント動作　停止
  if( tone != 0 ) {
    i = 12000000 / tone;                // i = tone/(1/(48e6/DIV4))
    GTIOC2A = tone / 2 - 1;             // 圧電スピーカーのON幅の設定
    startPWM_GPT2( GTIOCA , DIV4, i - 1 );  // ステアリングPWMの周期設定(GPT2 GTIOCAを使用)
  }
}

//**********************************************************************
// end of file
//**********************************************************************
